"""Fallback primitive operations that operate on 'object' operands.

These just call the relevant Python C API function or a thin wrapper
around an API function. Most of these also have faster, specialized
ops that operate on some more specific types.

Many of these ops are given a low priority (0) so that specialized ops
will take precedence. If your specialized op doesn't seem to be used,
check that the priorities are configured properly.
"""

from __future__ import annotations

from mypyc.ir.ops import ERR_MAGIC, ERR_NEVER
from mypyc.ir.rtypes import (
    bool_rprimitive,
    c_int_rprimitive,
    c_pyssize_t_rprimitive,
    c_size_t_rprimitive,
    int_rprimitive,
    object_pointer_rprimitive,
    object_rprimitive,
    pointer_rprimitive,
)
from mypyc.primitives.registry import (
    ERR_NEG_INT,
    binary_op,
    custom_op,
    function_op,
    method_op,
    unary_op,
)

# Binary operations

for op, opid in [
    ("==", 2),  # PY_EQ
    ("!=", 3),  # PY_NE
    ("<", 0),  # PY_LT
    ("<=", 1),  # PY_LE
    (">", 4),  # PY_GT
    (">=", 5),
]:  # PY_GE
    # The result type is 'object' since that's what PyObject_RichCompare returns.
    binary_op(
        name=op,
        arg_types=[object_rprimitive, object_rprimitive],
        return_type=object_rprimitive,
        c_function_name="PyObject_RichCompare",
        error_kind=ERR_MAGIC,
        extra_int_constants=[(opid, c_int_rprimitive)],
        priority=0,
    )

for op, funcname in [
    ("+", "PyNumber_Add"),
    ("-", "PyNumber_Subtract"),
    ("*", "PyNumber_Multiply"),
    ("//", "PyNumber_FloorDivide"),
    ("/", "PyNumber_TrueDivide"),
    ("%", "PyNumber_Remainder"),
    ("<<", "PyNumber_Lshift"),
    (">>", "PyNumber_Rshift"),
    ("&", "PyNumber_And"),
    ("^", "PyNumber_Xor"),
    ("|", "PyNumber_Or"),
    ("@", "PyNumber_MatrixMultiply"),
]:
    binary_op(
        name=op,
        arg_types=[object_rprimitive, object_rprimitive],
        return_type=object_rprimitive,
        c_function_name=funcname,
        error_kind=ERR_MAGIC,
        priority=0,
    )


function_op(
    name="builtins.divmod",
    arg_types=[object_rprimitive, object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="PyNumber_Divmod",
    error_kind=ERR_MAGIC,
    priority=0,
)


for op, funcname in [
    ("+=", "PyNumber_InPlaceAdd"),
    ("-=", "PyNumber_InPlaceSubtract"),
    ("*=", "PyNumber_InPlaceMultiply"),
    ("@=", "PyNumber_InPlaceMatrixMultiply"),
    ("//=", "PyNumber_InPlaceFloorDivide"),
    ("/=", "PyNumber_InPlaceTrueDivide"),
    ("%=", "PyNumber_InPlaceRemainder"),
    ("<<=", "PyNumber_InPlaceLshift"),
    (">>=", "PyNumber_InPlaceRshift"),
    ("&=", "PyNumber_InPlaceAnd"),
    ("^=", "PyNumber_InPlaceXor"),
    ("|=", "PyNumber_InPlaceOr"),
]:
    binary_op(
        name=op,
        arg_types=[object_rprimitive, object_rprimitive],
        return_type=object_rprimitive,
        c_function_name=funcname,
        error_kind=ERR_MAGIC,
        priority=0,
    )

for op, c_function in (("**", "CPyNumber_Power"), ("**=", "CPyNumber_InPlacePower")):
    binary_op(
        name=op,
        arg_types=[object_rprimitive, object_rprimitive],
        return_type=object_rprimitive,
        error_kind=ERR_MAGIC,
        c_function_name=c_function,
        priority=0,
    )

for arg_count, c_function in ((2, "CPyNumber_Power"), (3, "PyNumber_Power")):
    function_op(
        name="builtins.pow",
        arg_types=[object_rprimitive] * arg_count,
        return_type=object_rprimitive,
        error_kind=ERR_MAGIC,
        c_function_name=c_function,
        priority=0,
    )

binary_op(
    name="in",
    arg_types=[object_rprimitive, object_rprimitive],
    return_type=c_int_rprimitive,
    c_function_name="PySequence_Contains",
    error_kind=ERR_NEG_INT,
    truncated_type=bool_rprimitive,
    ordering=[1, 0],
    priority=0,
)


# Unary operations

for op, funcname in [
    ("-", "PyNumber_Negative"),
    ("+", "PyNumber_Positive"),
    ("~", "PyNumber_Invert"),
]:
    unary_op(
        name=op,
        arg_type=object_rprimitive,
        return_type=object_rprimitive,
        c_function_name=funcname,
        error_kind=ERR_MAGIC,
        priority=0,
    )

unary_op(
    name="not",
    arg_type=object_rprimitive,
    return_type=c_int_rprimitive,
    c_function_name="PyObject_Not",
    error_kind=ERR_NEG_INT,
    truncated_type=bool_rprimitive,
    priority=0,
)

# abs(obj)
function_op(
    name="builtins.abs",
    arg_types=[object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="PyNumber_Absolute",
    error_kind=ERR_MAGIC,
    priority=0,
)

# obj1[obj2]
py_get_item_op = method_op(
    name="__getitem__",
    arg_types=[object_rprimitive, object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="PyObject_GetItem",
    error_kind=ERR_MAGIC,
    priority=0,
)

# obj1[obj2] = obj3
method_op(
    name="__setitem__",
    arg_types=[object_rprimitive, object_rprimitive, object_rprimitive],
    return_type=c_int_rprimitive,
    c_function_name="PyObject_SetItem",
    error_kind=ERR_NEG_INT,
    priority=0,
)

# del obj1[obj2]
method_op(
    name="__delitem__",
    arg_types=[object_rprimitive, object_rprimitive],
    return_type=c_int_rprimitive,
    c_function_name="PyObject_DelItem",
    error_kind=ERR_NEG_INT,
    priority=0,
)

# hash(obj)
function_op(
    name="builtins.hash",
    arg_types=[object_rprimitive],
    return_type=int_rprimitive,
    c_function_name="CPyObject_Hash",
    error_kind=ERR_MAGIC,
)

# getattr(obj, attr)
py_getattr_op = function_op(
    name="builtins.getattr",
    arg_types=[object_rprimitive, object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="CPyObject_GetAttr",
    error_kind=ERR_MAGIC,
)

# getattr(obj, attr, default)
function_op(
    name="builtins.getattr",
    arg_types=[object_rprimitive, object_rprimitive, object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="CPyObject_GetAttr3",
    error_kind=ERR_MAGIC,
)

# setattr(obj, attr, value)
py_setattr_op = function_op(
    name="builtins.setattr",
    arg_types=[object_rprimitive, object_rprimitive, object_rprimitive],
    return_type=c_int_rprimitive,
    c_function_name="PyObject_SetAttr",
    error_kind=ERR_NEG_INT,
)

# hasattr(obj, attr)
py_hasattr_op = function_op(
    name="builtins.hasattr",
    arg_types=[object_rprimitive, object_rprimitive],
    return_type=bool_rprimitive,
    c_function_name="PyObject_HasAttr",
    error_kind=ERR_NEVER,
)

# del obj.attr
py_delattr_op = function_op(
    name="builtins.delattr",
    arg_types=[object_rprimitive, object_rprimitive],
    return_type=c_int_rprimitive,
    c_function_name="PyObject_DelAttr",
    error_kind=ERR_NEG_INT,
)

# Call callable object with N positional arguments: func(arg1, ..., argN)
# Arguments are (func, arg1, ..., argN).
py_call_op = custom_op(
    arg_types=[],
    return_type=object_rprimitive,
    c_function_name="PyObject_CallFunctionObjArgs",
    error_kind=ERR_MAGIC,
    var_arg_type=object_rprimitive,
    extra_int_constants=[(0, pointer_rprimitive)],
)

# Call callable object using positional and/or keyword arguments (Python 3.8+)
py_vectorcall_op = custom_op(
    arg_types=[
        object_rprimitive,  # Callable
        object_pointer_rprimitive,  # Args (PyObject **)
        c_size_t_rprimitive,  # Number of positional args
        object_rprimitive,
    ],  # Keyword arg names tuple (or NULL)
    return_type=object_rprimitive,
    c_function_name="PyObject_Vectorcall",
    error_kind=ERR_MAGIC,
)

# Call method using positional and/or keyword arguments (Python 3.9+)
py_vectorcall_method_op = custom_op(
    arg_types=[
        object_rprimitive,  # Method name
        object_pointer_rprimitive,  # Args, including self (PyObject **)
        c_size_t_rprimitive,  # Number of positional args, including self
        object_rprimitive,
    ],  # Keyword arg names tuple (or NULL)
    return_type=object_rprimitive,
    c_function_name="PyObject_VectorcallMethod",
    error_kind=ERR_MAGIC,
)

# Call callable object with positional + keyword args: func(*args, **kwargs)
# Arguments are (func, *args tuple, **kwargs dict).
py_call_with_kwargs_op = custom_op(
    arg_types=[object_rprimitive, object_rprimitive, object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="PyObject_Call",
    error_kind=ERR_MAGIC,
)

# Call method with positional arguments: obj.method(arg1, ...)
# Arguments are (object, attribute name, arg1, ...).
py_method_call_op = custom_op(
    arg_types=[],
    return_type=object_rprimitive,
    c_function_name="CPyObject_CallMethodObjArgs",
    error_kind=ERR_MAGIC,
    var_arg_type=object_rprimitive,
    extra_int_constants=[(0, pointer_rprimitive)],
)

# len(obj)
generic_len_op = custom_op(
    arg_types=[object_rprimitive],
    return_type=int_rprimitive,
    c_function_name="CPyObject_Size",
    error_kind=ERR_MAGIC,
)

# len(obj)
# same as generic_len_op, however return py_ssize_t
generic_ssize_t_len_op = custom_op(
    arg_types=[object_rprimitive],
    return_type=c_pyssize_t_rprimitive,
    c_function_name="PyObject_Size",
    error_kind=ERR_NEG_INT,
)

# iter(obj)
iter_op = function_op(
    name="builtins.iter",
    arg_types=[object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="PyObject_GetIter",
    error_kind=ERR_MAGIC,
)
# next(iterator)
#
# Although the error_kind is set to be ERR_NEVER, this can actually
# return NULL, and thus it must be checked using Branch.IS_ERROR.
next_op = custom_op(
    arg_types=[object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="PyIter_Next",
    error_kind=ERR_NEVER,
)
# next(iterator)
#
# Do a next, don't swallow StopIteration, but also don't propagate an
# error. (N.B: This can still return NULL without an error to
# represent an implicit StopIteration, but if StopIteration is
# *explicitly* raised this will not swallow it.)
# Can return NULL: see next_op.
next_raw_op = custom_op(
    arg_types=[object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="CPyIter_Next",
    error_kind=ERR_NEVER,
)

# this would be aiter(obj) if it existed
aiter_op = custom_op(
    arg_types=[object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="CPy_GetAIter",
    error_kind=ERR_MAGIC,
)

# this would be anext(obj) if it existed
anext_op = custom_op(
    arg_types=[object_rprimitive],
    return_type=object_rprimitive,
    c_function_name="CPy_GetANext",
    error_kind=ERR_MAGIC,
)



"""
PyCloud OS Event Bus
Sistem modülleri ve uygulamalar arasında olay tabanlı iletişim
"""

import logging
import threading
import queue
from typing import Dict, List, Callable, Any, Optional
from datetime import datetime
from enum import Enum

class EventPriority(Enum):
    """Olay öncelik seviyeleri"""
    LOW = 1
    NORMAL = 2
    HIGH = 3
    CRITICAL = 4

class Event:
    """Sistem olayı"""
    
    def __init__(self, event_type: str, data: Dict = None, 
                 priority: EventPriority = EventPriority.NORMAL,
                 source: str = None):
        self.event_type = event_type
        self.data = data or {}
        self.priority = priority
        self.source = source
        self.timestamp = datetime.now()
        self.handled = False
    
    def __str__(self):
        return f"Event({self.event_type}, {self.source}, {self.priority.name})"

class EventBus:
    """Olay yöneticisi - Pub/Sub modeli"""
    
    def __init__(self):
        self.logger = logging.getLogger("Events")
        self.subscribers: Dict[str, List[Callable]] = {}
        self.event_queue = queue.PriorityQueue()
        self.running = False
        self.worker_thread = None
        self.event_history: List[Event] = []
        self.max_history = 1000
        self._lock = threading.Lock()
        
        # İstatistikler
        self.stats = {
            "events_published": 0,
            "events_handled": 0,
            "subscribers_count": 0
        }
    
    def start(self):
        """Event bus'ı başlat"""
        if self.running:
            return
        
        self.running = True
        self.worker_thread = threading.Thread(target=self._event_worker, daemon=True)
        self.worker_thread.start()
        self.logger.info("Event bus started")
    
    def stop(self):
        """Event bus'ı durdur"""
        if not self.running:
            return
        
        self.running = False
        
        # Boş event ekleyerek worker thread'i uyandır
        self.event_queue.put((0, Event("__SHUTDOWN__")))
        
        if self.worker_thread and self.worker_thread.is_alive():
            self.worker_thread.join(timeout=1.0)
        
        self.logger.info("Event bus stopped")
    
    def subscribe(self, event_type: str, callback: Callable):
        """Olaya abone ol"""
        with self._lock:
            if event_type not in self.subscribers:
                self.subscribers[event_type] = []
            
            if callback not in self.subscribers[event_type]:
                self.subscribers[event_type].append(callback)
                self.stats["subscribers_count"] += 1
                self.logger.debug(f"Subscribed to {event_type}")
    
    def unsubscribe(self, event_type: str, callback: Callable):
        """Abonelikten çık"""
        with self._lock:
            if event_type in self.subscribers:
                if callback in self.subscribers[event_type]:
                    self.subscribers[event_type].remove(callback)
                    self.stats["subscribers_count"] -= 1
                    self.logger.debug(f"Unsubscribed from {event_type}")
                
                # Boş liste ise sil
                if not self.subscribers[event_type]:
                    del self.subscribers[event_type]
    
    def publish(self, event_type: str, data: Dict = None, 
                priority: EventPriority = EventPriority.NORMAL,
                source: str = None):
        """Olay yayınla"""
        event = Event(event_type, data, priority, source)
        
        # Önceliği tersine çevir (queue düşük sayıyı öncelikli görür)
        priority_value = 5 - priority.value
        
        self.event_queue.put((priority_value, event))
        self.stats["events_published"] += 1
        
        self.logger.debug(f"Published event: {event}")
    
    def publish_sync(self, event_type: str, data: Dict = None, source: str = None):
        """Senkron olay yayınla (anında işle)"""
        event = Event(event_type, data, EventPriority.CRITICAL, source)
        self._handle_event(event)
    
    def _event_worker(self):
        """Olay işleyici worker thread"""
        while self.running:
            try:
                # Timeout ile event bekle
                priority, event = self.event_queue.get(timeout=1.0)
                
                # Shutdown eventi
                if event.event_type == "__SHUTDOWN__":
                    break
                
                self._handle_event(event)
                self.event_queue.task_done()
                
            except queue.Empty:
                continue
            except Exception as e:
                self.logger.error(f"Event worker error: {e}")
    
    def _handle_event(self, event: Event):
        """Olayı işle"""
        try:
            # Geçmişe ekle
            self._add_to_history(event)
            
            # Aboneleri bul
            subscribers = []
            with self._lock:
                subscribers = self.subscribers.get(event.event_type, []).copy()
            
            # Aboneleri bilgilendir
            for callback in subscribers:
                try:
                    callback(event)
                except Exception as e:
                    self.logger.error(f"Event handler error: {e}")
            
            event.handled = True
            self.stats["events_handled"] += 1
            
            self.logger.debug(f"Handled event: {event} ({len(subscribers)} subscribers)")
            
        except Exception as e:
            self.logger.error(f"Failed to handle event {event}: {e}")
    
    def _add_to_history(self, event: Event):
        """Olayı geçmişe ekle"""
        self.event_history.append(event)
        
        # Geçmiş boyutunu sınırla
        if len(self.event_history) > self.max_history:
            self.event_history = self.event_history[-self.max_history:]
    
    def get_subscribers(self, event_type: str = None) -> Dict:
        """Aboneleri listele"""
        with self._lock:
            if event_type:
                return {event_type: len(self.subscribers.get(event_type, []))}
            else:
                return {et: len(subs) for et, subs in self.subscribers.items()}
    
    def get_history(self, event_type: str = None, limit: int = 100) -> List[Event]:
        """Olay geçmişini al"""
        if event_type:
            filtered = [e for e in self.event_history if e.event_type == event_type]
            return filtered[-limit:]
        else:
            return self.event_history[-limit:]
    
    def get_stats(self) -> Dict:
        """İstatistikleri al"""
        return {
            **self.stats,
            "queue_size": self.event_queue.qsize(),
            "history_size": len(self.event_history),
            "event_types": list(self.subscribers.keys())
        }
    
    def clear_history(self):
        """Geçmişi temizle"""
        self.event_history.clear()
        self.logger.info("Event history cleared")
    
    def shutdown(self):
        """Modül kapatma"""
        self.stop()

# Sistem olayları için sabitler
class SystemEvents:
    """Sistem olay türleri"""
    
    # Kernel olayları
    SYSTEM_BOOT = "system.boot"
    SYSTEM_SHUTDOWN = "system.shutdown"
    SYSTEM_RESTART = "system.restart"
    
    # Kullanıcı olayları
    USER_LOGIN = "user.login"
    USER_LOGOUT = "user.logout"
    USER_SWITCH = "user.switch"
    
    # Uygulama olayları
    APP_LAUNCH = "app.launch"
    APP_CLOSE = "app.close"
    APP_INSTALL = "app.install"
    APP_UNINSTALL = "app.uninstall"
    
    # UI olayları
    THEME_CHANGE = "ui.theme_change"
    WALLPAPER_CHANGE = "ui.wallpaper_change"
    DOCK_UPDATE = "ui.dock_update"
    
    # Dosya sistemi olayları
    FILE_CREATE = "fs.file_create"
    FILE_DELETE = "fs.file_delete"
    FILE_MODIFY = "fs.file_modify"
    FILE_MOVE = "fs.file_move"
    
    # Güvenlik olayları
    SECURITY_ALERT = "security.alert"
    LOGIN_ATTEMPT = "security.login_attempt"
    PERMISSION_DENIED = "security.permission_denied"
    
    # Bildirim olayları
    NOTIFICATION_SHOW = "notification.show"
    NOTIFICATION_CLICK = "notification.click"
    NOTIFICATION_DISMISS = "notification.dismiss"

# Kolaylık fonksiyonları
_global_event_bus: Optional[EventBus] = None

def get_event_bus() -> EventBus:
    """Global event bus'ı al"""
    global _global_event_bus
    if _global_event_bus is None:
        _global_event_bus = EventBus()
        _global_event_bus.start()
    return _global_event_bus

def publish(event_type: str, data: Dict = None, 
           priority: EventPriority = EventPriority.NORMAL,
           source: str = None):
    """Global event bus'a olay yayınla"""
    get_event_bus().publish(event_type, data, priority, source)

def subscribe(event_type: str, callback: Callable):
    """Global event bus'a abone ol"""
    get_event_bus().subscribe(event_type, callback)

def unsubscribe(event_type: str, callback: Callable):
    """Global event bus'tan abonelikten çık"""
    get_event_bus().unsubscribe(event_type, callback)

# Global events init function
def init_events(kernel=None) -> EventBus:
    """Events system'ı başlat"""
    event_bus = get_event_bus()
    return event_bus 


"""
Cloud Task Manager - Modern Ana Uygulama
macOS Activity Monitor tarzı sistem görev yöneticisi
"""

import os
import sys
import json
import logging
from pathlib import Path
from typing import Dict, List, Optional, Any
from datetime import datetime

try:
    from PyQt6.QtWidgets import *
    from PyQt6.QtCore import *
    from PyQt6.QtGui import *
except ImportError:
    print("PyQt6 not available for Cloud Task Manager")
    sys.exit(1)

from .system_monitor import SystemMonitor
from .process_manager import ProcessManager
from .thread_manager import ThreadManager
from .widgets import ModernTabWidget, SystemOverviewWidget, ProcessTableWidget, ThreadTableWidget, ResourceGraphWidget

class CloudTaskManager(QMainWindow):
    """Modern Cloud Task Manager Ana Uygulaması"""
    
    def __init__(self, kernel=None):
        super().__init__()
        self.kernel = kernel
        self.logger = logging.getLogger("CloudTaskManager")
        
        # Core managers
        self.system_monitor = SystemMonitor()
        self.process_manager = ProcessManager(kernel)
        self.thread_manager = ThreadManager(kernel)
        
        # Update timer
        self.update_timer = QTimer()
        self.update_timer.timeout.connect(self.update_all_data)
        self.update_interval = 2000  # 2 saniye
        
        # UI kurulumu
        self.setup_ui()
        self.setup_connections()
        self.apply_theme()
        
        # Monitoring başlat
        self.start_monitoring()
        
        self.logger.info("Cloud Task Manager v2.0.0 initialized")
    
    def setup_ui(self):
        """Modern UI kurulumu"""
        self.setWindowTitle("Cloud Task Manager")
        self.setGeometry(100, 100, 1200, 800)
        self.setMinimumSize(900, 600)
        
        # Ana widget
        central_widget = QWidget()
        self.setCentralWidget(central_widget)
        
        # Ana layout
        main_layout = QVBoxLayout(central_widget)
        main_layout.setSpacing(0)
        main_layout.setContentsMargins(0, 0, 0, 0)
        
        # Sistem özeti (üst panel)
        self.system_overview = SystemOverviewWidget()
        main_layout.addWidget(self.system_overview)
        
        # Ana tab widget
        self.tab_widget = ModernTabWidget()
        self.setup_tabs()
        main_layout.addWidget(self.tab_widget, 1)
        
        # Alt panel (kontroller)
        self.setup_control_panel()
        main_layout.addWidget(self.control_panel)
        
        # Toolbar
        self.setup_toolbar()
        
        # Status bar
        self.setup_statusbar()
        
        # Modern stil uygula
        self.apply_modern_style()
    
    def setup_tabs(self):
        """Sekmeleri kur - .cursorrules uyumlu"""
        # Applications sekmesi
        self.applications_widget = QWidget()
        self.setup_applications_tab()
        self.tab_widget.addTab(self.applications_widget, "🚀 Applications")
        
        # Processes sekmesi
        self.processes_widget = QWidget()
        self.setup_processes_tab()
        self.tab_widget.addTab(self.processes_widget, "⚙️ Processes")
        
        # Threads sekmesi
        self.threads_widget = QWidget()
        self.setup_threads_tab()
        self.tab_widget.addTab(self.threads_widget, "🧵 Threads")
        
        # Resources sekmesi
        self.resources_widget = QWidget()
        self.setup_resources_tab()
        self.tab_widget.addTab(self.resources_widget, "📊 Resources")
    
    def setup_applications_tab(self):
        """Applications sekmesi - PyCloud uygulamaları"""
        layout = QVBoxLayout(self.applications_widget)
        layout.setContentsMargins(20, 20, 20, 20)
        
        # Başlık ve açıklama
        title_label = QLabel("Running Applications")
        title_label.setObjectName("tabTitle")
        layout.addWidget(title_label)
        
        desc_label = QLabel("PyCloud OS uygulamaları ve durumları")
        desc_label.setObjectName("tabDescription")
        layout.addWidget(desc_label)
        
        # Filtre ve arama
        filter_layout = QHBoxLayout()
        
        self.app_search = QLineEdit()
        self.app_search.setPlaceholderText("🔍 Uygulama ara...")
        self.app_search.textChanged.connect(self.filter_applications)
        filter_layout.addWidget(self.app_search)
        
        self.app_status_filter = QComboBox()
        self.app_status_filter.addItems(["All", "Running", "Suspended", "Stopped"])
        self.app_status_filter.currentTextChanged.connect(self.filter_applications)
        filter_layout.addWidget(self.app_status_filter)
        
        layout.addLayout(filter_layout)
        
        # Uygulama tablosu
        self.app_table = ProcessTableWidget("applications")
        self.app_table.setColumnCount(6)
        self.app_table.setHorizontalHeaderLabels([
            "Application", "Status", "PID", "Memory", "CPU %", "Threads"
        ])
        
        # Tablo ayarları
        header = self.app_table.horizontalHeader()
        header.setStretchLastSection(True)
        header.setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
        
        layout.addWidget(self.app_table, 1)
    
    def setup_processes_tab(self):
        """Processes sekmesi - Sistem işlemleri"""
        layout = QVBoxLayout(self.processes_widget)
        layout.setContentsMargins(20, 20, 20, 20)
        
        # Başlık
        title_label = QLabel("System Processes")
        title_label.setObjectName("tabTitle")
        layout.addWidget(title_label)
        
        desc_label = QLabel("Tüm sistem işlemleri ve process ağacı")
        desc_label.setObjectName("tabDescription")
        layout.addWidget(desc_label)
        
        # Filtre ve kontroller
        filter_layout = QHBoxLayout()
        
        self.process_search = QLineEdit()
        self.process_search.setPlaceholderText("🔍 İşlem ara...")
        self.process_search.textChanged.connect(self.filter_processes)
        filter_layout.addWidget(self.process_search)
        
        self.process_sort = QComboBox()
        self.process_sort.addItems(["CPU", "Memory", "PID", "Name"])
        self.process_sort.currentTextChanged.connect(self.sort_processes)
        filter_layout.addWidget(self.process_sort)
        
        self.show_system_cb = QCheckBox("Show System Processes")
        self.show_system_cb.setChecked(True)
        self.show_system_cb.toggled.connect(self.filter_processes)
        filter_layout.addWidget(self.show_system_cb)
        
        layout.addLayout(filter_layout)
        
        # İşlem tablosu
        self.process_table = ProcessTableWidget("processes")
        self.process_table.setColumnCount(8)
        self.process_table.setHorizontalHeaderLabels([
            "PID", "Name", "Status", "CPU %", "Memory", "Threads", "User", "Command"
        ])
        
        # Bağlam menüsü
        self.process_table.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self.process_table.customContextMenuRequested.connect(self.show_process_context_menu)
        
        layout.addWidget(self.process_table, 1)
    
    def setup_threads_tab(self):
        """Threads sekmesi - Thread izleme"""
        layout = QVBoxLayout(self.threads_widget)
        layout.setContentsMargins(20, 20, 20, 20)
        
        # Başlık
        title_label = QLabel("Thread Monitor")
        title_label.setObjectName("tabTitle")
        layout.addWidget(title_label)
        
        desc_label = QLabel("Aktif thread'ler ve TID bazlı görünüm")
        desc_label.setObjectName("tabDescription")
        layout.addWidget(desc_label)
        
        # Thread kontrolleri
        control_layout = QHBoxLayout()
        
        self.thread_search = QLineEdit()
        self.thread_search.setPlaceholderText("🔍 Thread ara...")
        self.thread_search.textChanged.connect(self.filter_threads)
        control_layout.addWidget(self.thread_search)
        
        self.thread_process_filter = QComboBox()
        self.thread_process_filter.addItem("All Processes")
        self.thread_process_filter.currentTextChanged.connect(self.filter_threads)
        control_layout.addWidget(self.thread_process_filter)
        
        layout.addLayout(control_layout)
        
        # Thread tablosu
        self.thread_table = ThreadTableWidget()
        self.thread_table.setColumnCount(7)
        self.thread_table.setHorizontalHeaderLabels([
            "TID", "Process", "PID", "Status", "CPU %", "Priority", "State"
        ])
        
        # Bağlam menüsü
        self.thread_table.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self.thread_table.customContextMenuRequested.connect(self.show_thread_context_menu)
        
        layout.addWidget(self.thread_table, 1)
    
    def setup_resources_tab(self):
        """Resources sekmesi - Kaynak grafikleri"""
        layout = QVBoxLayout(self.resources_widget)
        layout.setContentsMargins(20, 20, 20, 20)
        
        # Başlık
        title_label = QLabel("System Resources")
        title_label.setObjectName("tabTitle")
        layout.addWidget(title_label)
        
        desc_label = QLabel("Gerçek zamanlı sistem kaynak kullanımı")
        desc_label.setObjectName("tabDescription")
        layout.addWidget(desc_label)
        
        # Kaynak grafikleri
        graphs_layout = QGridLayout()
        
        # CPU grafiği
        self.cpu_graph = ResourceGraphWidget("CPU Usage", "%", max_value=100)
        graphs_layout.addWidget(self.cpu_graph, 0, 0)
        
        # Memory grafiği
        self.memory_graph = ResourceGraphWidget("Memory Usage", "GB", max_value=16)
        graphs_layout.addWidget(self.memory_graph, 0, 1)
        
        # Disk I/O grafiği
        self.disk_graph = ResourceGraphWidget("Disk I/O", "MB/s", max_value=100)
        graphs_layout.addWidget(self.disk_graph, 1, 0)
        
        # Network grafiği
        self.network_graph = ResourceGraphWidget("Network", "MB/s", max_value=100)
        graphs_layout.addWidget(self.network_graph, 1, 1)
        
        layout.addLayout(graphs_layout, 1)
        
        # Detaylı sistem bilgileri
        self.setup_system_info_panel()
        layout.addWidget(self.system_info_panel)
    
    def setup_system_info_panel(self):
        """Sistem bilgi paneli"""
        self.system_info_panel = QGroupBox("System Information")
        layout = QGridLayout(self.system_info_panel)
        
        # Sistem bilgileri
        self.cpu_cores_label = QLabel("CPU Cores: --")
        self.memory_total_label = QLabel("Total Memory: --")
        self.uptime_label = QLabel("Uptime: --")
        self.processes_count_label = QLabel("Processes: --")
        
        layout.addWidget(self.cpu_cores_label, 0, 0)
        layout.addWidget(self.memory_total_label, 0, 1)
        layout.addWidget(self.uptime_label, 1, 0)
        layout.addWidget(self.processes_count_label, 1, 1)
    
    def setup_control_panel(self):
        """Alt kontrol paneli"""
        self.control_panel = QWidget()
        self.control_panel.setFixedHeight(60)
        
        layout = QHBoxLayout(self.control_panel)
        layout.setContentsMargins(20, 10, 20, 10)
        
        # Sol taraf - eylem butonları
        self.end_task_btn = QPushButton("End Task")
        self.end_task_btn.setObjectName("dangerButton")
        self.end_task_btn.clicked.connect(self.end_selected_task)
        self.end_task_btn.setEnabled(False)
        layout.addWidget(self.end_task_btn)
        
        self.force_quit_btn = QPushButton("Force Quit")
        self.force_quit_btn.setObjectName("dangerButton")
        self.force_quit_btn.clicked.connect(self.force_quit_selected)
        self.force_quit_btn.setEnabled(False)
        layout.addWidget(self.force_quit_btn)
        
        self.suspend_btn = QPushButton("Suspend")
        self.suspend_btn.clicked.connect(self.suspend_selected)
        self.suspend_btn.setEnabled(False)
        layout.addWidget(self.suspend_btn)
        
        layout.addStretch()
        
        # Sağ taraf - ayarlar
        self.auto_refresh_cb = QCheckBox("Auto Refresh")
        self.auto_refresh_cb.setChecked(True)
        self.auto_refresh_cb.toggled.connect(self.toggle_auto_refresh)
        layout.addWidget(self.auto_refresh_cb)
        
        self.refresh_btn = QPushButton("🔄 Refresh")
        self.refresh_btn.clicked.connect(self.manual_refresh)
        layout.addWidget(self.refresh_btn)
        
        # Update interval
        interval_layout = QHBoxLayout()
        interval_layout.addWidget(QLabel("Update:"))
        
        self.interval_combo = QComboBox()
        self.interval_combo.addItems(["1s", "2s", "5s", "10s"])
        self.interval_combo.setCurrentText("2s")
        self.interval_combo.currentTextChanged.connect(self.change_update_interval)
        interval_layout.addWidget(self.interval_combo)
        
        layout.addLayout(interval_layout)
    
    def setup_toolbar(self):
        """Toolbar kurulumu"""
        self.toolbar = self.addToolBar("Main")
        self.toolbar.setMovable(False)
        
        # Tema değiştirici
        theme_action = QAction("🌓", self)
        theme_action.setToolTip("Toggle Theme")
        theme_action.triggered.connect(self.toggle_theme)
        self.toolbar.addAction(theme_action)
        
        self.toolbar.addSeparator()
        
        # Ayarlar
        settings_action = QAction("⚙️", self)
        settings_action.setToolTip("Settings")
        settings_action.triggered.connect(self.show_settings)
        self.toolbar.addAction(settings_action)
        
        # Yardım
        help_action = QAction("❓", self)
        help_action.setToolTip("Help")
        help_action.triggered.connect(self.show_help)
        self.toolbar.addAction(help_action)
    
    def setup_statusbar(self):
        """Status bar kurulumu"""
        self.status_bar = self.statusBar()
        
        # Sol taraf - durum
        self.status_label = QLabel("Ready")
        self.status_bar.addWidget(self.status_label)
        
        # Sağ taraf - bilgiler
        self.update_status_label = QLabel("Last update: --")
        self.status_bar.addPermanentWidget(self.update_status_label)
        
        self.selected_count_label = QLabel("0 selected")
        self.status_bar.addPermanentWidget(self.selected_count_label)
    
    def setup_connections(self):
        """Sinyal bağlantıları"""
        # Tab değişimi
        self.tab_widget.currentChanged.connect(self.on_tab_changed)
        
        # Tablo seçimleri
        self.app_table.itemSelectionChanged.connect(self.on_selection_changed)
        self.process_table.itemSelectionChanged.connect(self.on_selection_changed)
        self.thread_table.itemSelectionChanged.connect(self.on_selection_changed)
    
    def apply_modern_style(self):
        """Modern stil uygula"""
        self.setStyleSheet("""
            /* Ana pencere */
            QMainWindow {
                background-color: #f8f9fa;
            }
            
            /* Tab başlıkları */
            QLabel#tabTitle {
                font-size: 24px;
                font-weight: 700;
                color: #212529;
                margin-bottom: 5px;
            }
            
            QLabel#tabDescription {
                font-size: 14px;
                color: #6c757d;
                margin-bottom: 20px;
            }
            
            /* Butonlar */
            QPushButton {
                background-color: #007bff;
                color: white;
                border: none;
                padding: 8px 16px;
                border-radius: 6px;
                font-weight: 600;
                font-size: 14px;
            }
            
            QPushButton:hover {
                background-color: #0056b3;
            }
            
            QPushButton:disabled {
                background-color: #e9ecef;
                color: #6c757d;
            }
            
            QPushButton#dangerButton {
                background-color: #dc3545;
            }
            
            QPushButton#dangerButton:hover {
                background-color: #c82333;
            }
            
            /* Tablolar */
            QTableWidget {
                border: 1px solid #dee2e6;
                border-radius: 8px;
                background-color: #ffffff;
                gridline-color: #e9ecef;
                selection-background-color: #007bff;
                selection-color: white;
            }
            
            QHeaderView::section {
                background-color: #f8f9fa;
                border: 1px solid #dee2e6;
                padding: 8px;
                font-weight: 600;
            }
            
            /* Kontrol paneli */
            QWidget#controlPanel {
                background-color: #ffffff;
                border-top: 1px solid #dee2e6;
            }
            
            /* Toolbar */
            QToolBar {
                background-color: #ffffff;
                border-bottom: 1px solid #dee2e6;
                spacing: 10px;
                padding: 8px;
            }
            
            QToolBar QToolButton {
                background-color: transparent;
                border: none;
                padding: 8px;
                border-radius: 6px;
                font-size: 16px;
            }
            
            QToolBar QToolButton:hover {
                background-color: #f8f9fa;
            }
            
            /* Status bar */
            QStatusBar {
                background-color: #ffffff;
                border-top: 1px solid #dee2e6;
                color: #6c757d;
            }
        """)
    
    def apply_theme(self):
        """Tema uygula"""
        # Şimdilik açık tema
        pass
    
    def start_monitoring(self):
        """Monitoring başlat"""
        self.system_monitor.start()
        self.process_manager.start()
        self.thread_manager.start()
        
        self.update_timer.start(self.update_interval)
        self.logger.info("Monitoring started")
    
    def stop_monitoring(self):
        """Monitoring durdur"""
        self.update_timer.stop()
        self.system_monitor.stop()
        self.process_manager.stop()
        self.thread_manager.stop()
        
        self.logger.info("Monitoring stopped")
    
    def update_all_data(self):
        """Tüm verileri güncelle"""
        try:
            # Sistem özeti güncelle
            self.update_system_overview()
            
            # Aktif sekmeye göre güncelle
            current_tab = self.tab_widget.currentIndex()
            
            if current_tab == 0:  # Applications
                self.update_applications()
            elif current_tab == 1:  # Processes
                self.update_processes()
            elif current_tab == 2:  # Threads
                self.update_threads()
            elif current_tab == 3:  # Resources
                self.update_resources()
            
            # Status bar güncelle
            self.update_status_label.setText(f"Last update: {datetime.now().strftime('%H:%M:%S')}")
            
        except Exception as e:
            self.logger.error(f"Update failed: {e}")
    
    def update_system_overview(self):
        """Sistem özeti güncelle"""
        stats = self.system_monitor.get_system_stats()
        self.system_overview.update_stats(stats)
    
    def update_applications(self):
        """Uygulamaları güncelle"""
        apps = self.process_manager.get_applications()
        self.populate_app_table(apps)
    
    def update_processes(self):
        """İşlemleri güncelle"""
        processes = self.process_manager.get_processes()
        self.populate_process_table(processes)
    
    def update_threads(self):
        """Thread'leri güncelle"""
        threads = self.thread_manager.get_threads()
        self.populate_thread_table(threads)
        
        # Process filter güncelle
        processes = list(set([t.process_name for t in threads]))
        current_text = self.thread_process_filter.currentText()
        self.thread_process_filter.clear()
        self.thread_process_filter.addItem("All Processes")
        self.thread_process_filter.addItems(sorted(processes))
        
        # Önceki seçimi geri yükle
        index = self.thread_process_filter.findText(current_text)
        if index >= 0:
            self.thread_process_filter.setCurrentIndex(index)
    
    def update_resources(self):
        """Kaynakları güncelle"""
        stats = self.system_monitor.get_system_stats()
        
        # Grafikleri güncelle
        self.cpu_graph.add_data_point(stats.cpu_percent)
        self.memory_graph.add_data_point(stats.memory_used_gb)
        self.disk_graph.add_data_point(stats.disk_io_mb)
        self.network_graph.add_data_point(stats.network_mb)
        
        # Sistem bilgileri güncelle
        self.cpu_cores_label.setText(f"CPU Cores: {stats.cpu_cores}")
        self.memory_total_label.setText(f"Total Memory: {stats.memory_total_gb:.1f} GB")
        self.uptime_label.setText(f"Uptime: {stats.uptime_hours:.1f} hours")
        self.processes_count_label.setText(f"Processes: {stats.process_count}")
    
    def populate_app_table(self, apps):
        """Uygulama tablosunu doldur"""
        self.app_table.setRowCount(len(apps))
        
        for row, app in enumerate(apps):
            self.app_table.setItem(row, 0, QTableWidgetItem(app.name))
            self.app_table.setItem(row, 1, QTableWidgetItem(app.status))
            self.app_table.setItem(row, 2, QTableWidgetItem(str(app.pid)))
            self.app_table.setItem(row, 3, QTableWidgetItem(f"{app.memory_mb:.1f} MB"))
            self.app_table.setItem(row, 4, QTableWidgetItem(f"{app.cpu_percent:.1f}%"))
            self.app_table.setItem(row, 5, QTableWidgetItem(str(app.thread_count)))
    
    def populate_process_table(self, processes):
        """İşlem tablosunu doldur"""
        self.process_table.setRowCount(len(processes))
        
        for row, proc in enumerate(processes):
            self.process_table.setItem(row, 0, QTableWidgetItem(str(proc.pid)))
            self.process_table.setItem(row, 1, QTableWidgetItem(proc.name))
            self.process_table.setItem(row, 2, QTableWidgetItem(proc.status))
            self.process_table.setItem(row, 3, QTableWidgetItem(f"{proc.cpu_percent:.1f}%"))
            self.process_table.setItem(row, 4, QTableWidgetItem(f"{proc.memory_mb:.1f} MB"))
            self.process_table.setItem(row, 5, QTableWidgetItem(str(proc.thread_count)))
            self.process_table.setItem(row, 6, QTableWidgetItem(proc.username))
            self.process_table.setItem(row, 7, QTableWidgetItem(proc.cmdline[:50] + "..." if len(proc.cmdline) > 50 else proc.cmdline))
    
    def populate_thread_table(self, threads):
        """Thread tablosunu doldur"""
        self.thread_table.setRowCount(len(threads))
        
        for row, thread in enumerate(threads):
            self.thread_table.setItem(row, 0, QTableWidgetItem(str(thread.tid)))
            self.thread_table.setItem(row, 1, QTableWidgetItem(thread.process_name))
            self.thread_table.setItem(row, 2, QTableWidgetItem(str(thread.pid)))
            self.thread_table.setItem(row, 3, QTableWidgetItem(thread.status))
            self.thread_table.setItem(row, 4, QTableWidgetItem(f"{thread.cpu_percent:.1f}%"))
            self.thread_table.setItem(row, 5, QTableWidgetItem(str(thread.priority)))
            self.thread_table.setItem(row, 6, QTableWidgetItem(thread.state))
    
    # Event handlers
    def on_tab_changed(self, index):
        """Sekme değişti"""
        tab_names = ["Applications", "Processes", "Threads", "Resources"]
        self.status_label.setText(f"Viewing {tab_names[index]}")
        
        # Hemen güncelle
        self.update_all_data()
    
    def on_selection_changed(self):
        """Seçim değişti"""
        current_table = self.get_current_table()
        if current_table:
            selected_count = len(current_table.selectedItems()) // current_table.columnCount()
            self.selected_count_label.setText(f"{selected_count} selected")
            
            # Butonları etkinleştir/devre dışı bırak
            has_selection = selected_count > 0
            self.end_task_btn.setEnabled(has_selection)
            self.force_quit_btn.setEnabled(has_selection)
            self.suspend_btn.setEnabled(has_selection)
    
    def get_current_table(self):
        """Aktif tabloyu al"""
        current_tab = self.tab_widget.currentIndex()
        
        if current_tab == 0:
            return self.app_table
        elif current_tab == 1:
            return self.process_table
        elif current_tab == 2:
            return self.thread_table
        
        return None
    
    # Filter methods
    def filter_applications(self):
        """Uygulamaları filtrele"""
        search_text = self.app_search.text().lower()
        status_filter = self.app_status_filter.currentText()
        
        for row in range(self.app_table.rowCount()):
            app_name = self.app_table.item(row, 0).text().lower()
            app_status = self.app_table.item(row, 1).text()
            
            name_match = search_text in app_name
            status_match = status_filter == "All" or status_filter == app_status
            
            self.app_table.setRowHidden(row, not (name_match and status_match))
    
    def filter_processes(self):
        """İşlemleri filtrele"""
        search_text = self.process_search.text().lower()
        show_system = self.show_system_cb.isChecked()
        
        for row in range(self.process_table.rowCount()):
            if self.process_table.item(row, 1):
                process_name = self.process_table.item(row, 1).text().lower()
                name_match = search_text in process_name
                
                # Sistem işlemi kontrolü (basit)
                is_system = process_name.startswith(('kernel', 'system', 'com.apple', 'launchd'))
                system_match = show_system or not is_system
                
                self.process_table.setRowHidden(row, not (name_match and system_match))
    
    def filter_threads(self):
        """Thread'leri filtrele"""
        search_text = self.thread_search.text().lower()
        process_filter = self.thread_process_filter.currentText()
        
        for row in range(self.thread_table.rowCount()):
            if self.thread_table.item(row, 1):
                process_name = self.thread_table.item(row, 1).text()
                
                name_match = search_text in process_name.lower()
                process_match = process_filter == "All Processes" or process_filter == process_name
                
                self.thread_table.setRowHidden(row, not (name_match and process_match))
    
    def sort_processes(self):
        """İşlemleri sırala"""
        sort_by = self.process_sort.currentText()
        
        if sort_by == "CPU":
            self.process_table.sortItems(3, Qt.SortOrder.DescendingOrder)
        elif sort_by == "Memory":
            self.process_table.sortItems(4, Qt.SortOrder.DescendingOrder)
        elif sort_by == "PID":
            self.process_table.sortItems(0, Qt.SortOrder.AscendingOrder)
        elif sort_by == "Name":
            self.process_table.sortItems(1, Qt.SortOrder.AscendingOrder)
    
    # Context menus
    def show_process_context_menu(self, position):
        """İşlem bağlam menüsü"""
        if self.process_table.itemAt(position) is None:
            return
        
        menu = QMenu()
        
        end_action = menu.addAction("End Process")
        end_action.triggered.connect(self.end_selected_task)
        
        force_action = menu.addAction("Force Quit")
        force_action.triggered.connect(self.force_quit_selected)
        
        menu.addSeparator()
        
        suspend_action = menu.addAction("Suspend")
        suspend_action.triggered.connect(self.suspend_selected)
        
        resume_action = menu.addAction("Resume")
        resume_action.triggered.connect(self.resume_selected)
        
        menu.addSeparator()
        
        properties_action = menu.addAction("Properties")
        properties_action.triggered.connect(self.show_process_properties)
        
        menu.exec(self.process_table.mapToGlobal(position))
    
    def show_thread_context_menu(self, position):
        """Thread bağlam menüsü"""
        if self.thread_table.itemAt(position) is None:
            return
        
        menu = QMenu()
        
        freeze_action = menu.addAction("Freeze Thread")
        freeze_action.triggered.connect(self.freeze_selected_thread)
        
        kill_action = menu.addAction("Kill Thread")
        kill_action.triggered.connect(self.kill_selected_thread)
        
        menu.addSeparator()
        
        properties_action = menu.addAction("Thread Properties")
        properties_action.triggered.connect(self.show_thread_properties)
        
        menu.exec(self.thread_table.mapToGlobal(position))
    
    # Action methods
    def end_selected_task(self):
        """Seçili görevi sonlandır"""
        current_table = self.get_current_table()
        if not current_table:
            return
        
        selected_rows = set()
        for item in current_table.selectedItems():
            selected_rows.add(item.row())
        
        if not selected_rows:
            return
        
        # Onay al
        reply = QMessageBox.question(
            self, "End Task",
            f"Are you sure you want to end {len(selected_rows)} task(s)?",
            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
        )
        
        if reply == QMessageBox.StandardButton.Yes:
            for row in selected_rows:
                if self.tab_widget.currentIndex() == 0:  # Applications
                    pid_item = current_table.item(row, 2)
                elif self.tab_widget.currentIndex() == 1:  # Processes
                    pid_item = current_table.item(row, 0)
                else:
                    continue
                
                if pid_item:
                    pid = int(pid_item.text())
                    self.process_manager.terminate_process(pid)
            
            self.status_label.setText(f"Ended {len(selected_rows)} task(s)")
    
    def force_quit_selected(self):
        """Seçili görevi zorla sonlandır"""
        # end_selected_task ile benzer ama force=True
        self.end_selected_task()  # Şimdilik aynı
    
    def suspend_selected(self):
        """Seçili görevi askıya al"""
        self.status_label.setText("Suspend functionality not implemented yet")
    
    def resume_selected(self):
        """Seçili görevi devam ettir"""
        self.status_label.setText("Resume functionality not implemented yet")
    
    def freeze_selected_thread(self):
        """Seçili thread'i dondur"""
        selected_rows = set()
        for item in self.thread_table.selectedItems():
            selected_rows.add(item.row())
        
        for row in selected_rows:
            tid_item = self.thread_table.item(row, 0)
            if tid_item:
                tid = int(tid_item.text())
                self.thread_manager.freeze_thread(tid)
        
        self.status_label.setText(f"Froze {len(selected_rows)} thread(s)")
    
    def kill_selected_thread(self):
        """Seçili thread'i öldür"""
        selected_rows = set()
        for item in self.thread_table.selectedItems():
            selected_rows.add(item.row())
        
        if not selected_rows:
            return
        
        reply = QMessageBox.question(
            self, "Kill Thread",
            f"Are you sure you want to kill {len(selected_rows)} thread(s)?",
            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
        )
        
        if reply == QMessageBox.StandardButton.Yes:
            for row in selected_rows:
                tid_item = self.thread_table.item(row, 0)
                if tid_item:
                    tid = int(tid_item.text())
                    self.thread_manager.kill_thread(tid)
            
            self.status_label.setText(f"Killed {len(selected_rows)} thread(s)")
    
    def show_process_properties(self):
        """İşlem özelliklerini göster"""
        self.status_label.setText("Process properties dialog not implemented yet")
    
    def show_thread_properties(self):
        """Thread özelliklerini göster"""
        self.status_label.setText("Thread properties dialog not implemented yet")
    
    # Control methods
    def toggle_auto_refresh(self, enabled):
        """Otomatik yenilemeyi aç/kapat"""
        if enabled:
            self.update_timer.start(self.update_interval)
            self.status_label.setText("Auto refresh enabled")
        else:
            self.update_timer.stop()
            self.status_label.setText("Auto refresh disabled")
    
    def manual_refresh(self):
        """Manuel yenile"""
        self.update_all_data()
        self.status_label.setText("Manually refreshed")
    
    def change_update_interval(self, interval_text):
        """Güncelleme aralığını değiştir"""
        interval_map = {"1s": 1000, "2s": 2000, "5s": 5000, "10s": 10000}
        self.update_interval = interval_map.get(interval_text, 2000)
        
        if self.update_timer.isActive():
            self.update_timer.setInterval(self.update_interval)
        
        self.status_label.setText(f"Update interval: {interval_text}")
    
    def toggle_theme(self):
        """Temayı değiştir"""
        self.status_label.setText("Theme toggle not implemented yet")
    
    def show_settings(self):
        """Ayarları göster"""
        self.status_label.setText("Settings dialog not implemented yet")
    
    def show_help(self):
        """Yardım göster"""
        help_text = """
        <h2>Cloud Task Manager v2.0.0</h2>
        
        <h3>🚀 Applications</h3>
        <p>PyCloud OS uygulamalarını görüntüleyin ve yönetin.</p>
        
        <h3>⚙️ Processes</h3>
        <p>Sistem işlemlerini izleyin ve kontrol edin.</p>
        
        <h3>🧵 Threads</h3>
        <p>Thread'leri TID bazlı görüntüleyin ve yönetin.</p>
        
        <h3>📊 Resources</h3>
        <p>Sistem kaynaklarını gerçek zamanlı izleyin.</p>
        
        <h3>Klavye Kısayolları</h3>
        <ul>
        <li><b>Ctrl+R:</b> Yenile</li>
        <li><b>Delete:</b> Seçili görevi sonlandır</li>
        <li><b>Ctrl+F:</b> Arama</li>
        </ul>
        """
        
        QMessageBox.about(self, "Help", help_text)
    
    def closeEvent(self, event):
        """Pencere kapatılıyor"""
        self.stop_monitoring()
        event.accept()
        self.logger.info("Cloud Task Manager closed") 


"""
Core Thread Package - PyCloud OS Thread Yönetimi
"""

from .queue import ThreadQueue
from .pool import ThreadPool
from .msg import ThreadMessaging
from .snapshot import ThreadSnapshot
from .profile import ThreadProfiler

# Ana thread_main.py modülünden fonksiyonları import et
import sys
from pathlib import Path

# Parent directory'yi path'e ekle
sys.path.insert(0, str(Path(__file__).parent.parent))

# thread_main.py'den gerekli fonksiyonları import et
from thread_main import ThreadManager, init_thread_manager, create_thread, start_thread, get_thread_manager

__all__ = [
    'ThreadQueue', 'ThreadPool', 'ThreadMessaging', 'ThreadSnapshot', 'ThreadProfiler',
    'ThreadManager', 'init_thread_manager', 'create_thread', 'start_thread', 'get_thread_manager'
] 


#!/usr/bin/env python3
"""
Cloud Browser v2.0.0 - Modern Web Tarayıcısı
PyCloud OS için geliştirilmiş sekmeli web tarayıcısı
"""

import sys
import os
import logging
from pathlib import Path

# PyQt6 import ve WebEngine için gerekli ayarlar
try:
    from PyQt6.QtWidgets import QApplication
    from PyQt6.QtCore import Qt, QCoreApplication
    from PyQt6.QtGui import QIcon
    
    # WebEngine için gerekli ayar - QApplication oluşturmadan önce
    QCoreApplication.setAttribute(Qt.ApplicationAttribute.AA_ShareOpenGLContexts)
    
except ImportError as e:
    print(f"❌ PyQt6 import hatası: {e}")
    print("💡 Çözüm: pip install PyQt6 PyQt6-WebEngine")
    sys.exit(1)

# Proje kök dizinini sys.path'e ekle
project_root = Path(__file__).parent.parent.parent
sys.path.insert(0, str(project_root))

# Local core modülü için path ekle
local_core = Path(__file__).parent
sys.path.insert(0, str(local_core))

def setup_logging():
    """Logging sistemini kur"""
    log_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(
        level=logging.INFO,
        format=log_format,
        handlers=[
            logging.StreamHandler(sys.stdout)
        ]
    )
    return logging.getLogger("CloudBrowser")

def check_dependencies():
    """Gerekli bağımlılıkları kontrol et"""
    try:
        from PyQt6.QtWebEngineWidgets import QWebEngineView
        return True
    except ImportError:
        return False

def main():
    """Ana fonksiyon"""
    logger = setup_logging()
    logger.info("🌐 Cloud Browser v2.0.0 starting...")
    
    # QApplication instance kontrolü
    app = QApplication.instance()
    if app is None:
        app = QApplication(sys.argv)
        app.setApplicationName("Cloud Browser")
        app.setApplicationVersion("2.0.0")
        app.setOrganizationName("PyCloud OS")
        
        # High DPI desteği (PyQt6 uyumlu)
        try:
            app.setAttribute(Qt.ApplicationAttribute.AA_EnableHighDpiScaling, True)
        except AttributeError:
            pass  # PyQt6'da bu ayar varsayılan olarak aktif
        
        try:
            app.setAttribute(Qt.ApplicationAttribute.AA_UseHighDpiPixmaps, True)
        except AttributeError:
            pass  # PyQt6'da bu ayar varsayılan olarak aktif
    
    # WebEngine kontrolü
    webengine_available = check_dependencies()
    if webengine_available:
        logger.info("✅ WebEngine available")
    else:
        logger.info("⚠️ WebEngine not available, using fallback mode")
    
    # Kernel bağlantısı (opsiyonel)
    kernel = None
    try:
        # PyCloud kernel'ı import etmeye çalış
        from core.kernel import PyCloudKernel
        kernel = PyCloudKernel()
        logger.info("✅ PyCloud kernel connected")
    except ImportError:
        logger.info("ℹ️ PyCloud kernel not available, running standalone")
    except Exception as e:
        logger.warning(f"⚠️ Kernel connection failed: {e}")
    
    # Browser oluştur ve göster
    try:
        logger.info("Creating CloudBrowser instance...")
        from core.browser_app import CloudBrowser
        
        browser = CloudBrowser(kernel)
        
        # Icon ayarla (varsa)
        icon_path = Path(__file__).parent / "icon.png"
        if icon_path.exists():
            browser.setWindowIcon(QIcon(str(icon_path)))
        
        logger.info("Showing browser window...")
        browser.show()
        
        logger.info("✅ Cloud Browser v2.0.0 started successfully")
        logger.info("Starting event loop...")
        
        # Event loop başlat
        exit_code = app.exec()
        
        logger.info(f"🔚 Cloud Browser exited with code: {exit_code}")
        return exit_code
        
    except Exception as e:
        logger.error(f"❌ Critical error: {e}")
        import traceback
        traceback.print_exc()
        return 1
    
    finally:
        logger.info("🧹 Cleaning up...")

if __name__ == "__main__":
    sys.exit(main()) 


"""
Cloud Terminal - Autocomplete Sistemi
Komut ve dosya tamamlama önerileri
"""

import os
from typing import List, Optional
from pathlib import Path

class AutoCompleter:
    """Autocomplete sistemi"""
    
    def __init__(self, command_runner=None):
        self.command_runner = command_runner
        
        # Sistem komutları
        self.system_commands = [
            'help', 'clear', 'exit', 'version', 'uptime', 'date', 'whoami'
        ]
        
        # Dosya komutları
        self.file_commands = [
            'pwd', 'cd', 'ls', 'dir', 'cat', 'head', 'tail', 'find',
            'mkdir', 'rmdir', 'rm', 'cp', 'mv', 'touch'
        ]
        
        # İşlem komutları
        self.process_commands = [
            'ps', 'kill', 'killall', 'top', 'mem', 'thread'
        ]
        
        # Kullanıcı komutları
        self.user_commands = [
            'users', 'su', 'profile'
        ]
        
        # Geliştirici komutları
        self.developer_commands = [
            'clapp', 'pycloud', 'python', 'python3', 'pip', 'git', 'notify', 'fs'
        ]
        
        # Tüm komutlar
        self.all_commands = (
            self.system_commands + 
            self.file_commands + 
            self.process_commands + 
            self.user_commands + 
            self.developer_commands
        )
        
        # Alt komutlar
        self.sub_commands = {
            'clapp': ['list', 'install', 'remove', 'update', 'search', 'info', 'doctor', 'upgrade', 'history', 'help'],
            'pycloud': ['status', 'restart', 'shutdown', 'modules'],
            'git': ['init', 'add', 'commit', 'push', 'pull', 'status', 'log'],
            'pip': ['install', 'uninstall', 'list', 'show', 'search'],
            'ls': ['-l', '-a', '-la', '--long', '--all'],
            'cd': ['..', '.', '~'],
        }
    
    def get_completions(self, partial_command: str, current_dir: str) -> List[str]:
        """Tamamlama önerilerini al"""
        if not partial_command.strip():
            return []
        
        parts = partial_command.split()
        
        if len(parts) == 1:
            # İlk kelime - komut tamamlama
            return self._complete_command(parts[0])
        else:
            # Sonraki kelimeler - bağlama göre tamamlama
            return self._complete_arguments(parts, current_dir)
    
    def _complete_command(self, partial: str) -> List[str]:
        """Komut tamamlama"""
        partial = partial.lower()
        matches = []
        
        # Komut eşleşmeleri
        for cmd in self.all_commands:
            if cmd.startswith(partial):
                matches.append(cmd)
        
        # Dosya/dizin eşleşmeleri (eğer / içeriyorsa)
        if '/' in partial or partial.startswith('.'):
            try:
                file_matches = self._complete_path(partial, os.getcwd())
                matches.extend(file_matches)
            except:
                pass
        
        return sorted(matches)
    
    def _complete_arguments(self, parts: List[str], current_dir: str) -> List[str]:
        """Argüman tamamlama"""
        command = parts[0].lower()
        current_arg = parts[-1] if len(parts) > 1 else ""
        
        # Alt komut tamamlama
        if command in self.sub_commands and len(parts) == 2:
            sub_cmds = self.sub_commands[command]
            matches = [cmd for cmd in sub_cmds if cmd.startswith(current_arg)]
            if matches:
                return matches
        
        # Dosya/dizin tamamlama
        if command in ['cd', 'ls', 'cat', 'head', 'tail', 'rm', 'cp', 'mv', 'mkdir']:
            return self._complete_path(current_arg, current_dir)
        
        # Özel tamamlamalar
        if command == 'clapp' and len(parts) == 3 and parts[1] in ['install', 'remove']:
            return self._complete_app_names()
        
        return []
    
    def _complete_path(self, partial: str, current_dir: str) -> List[str]:
        """Dosya/dizin yolu tamamlama"""
        try:
            if not partial:
                # Boş - mevcut dizindeki tüm öğeler
                target_dir = current_dir
                prefix = ""
            elif partial.startswith('/'):
                # Mutlak yol
                target_dir = os.path.dirname(partial) or '/'
                prefix = os.path.basename(partial)
            elif partial.startswith('~'):
                # Home dizini
                expanded = os.path.expanduser(partial)
                target_dir = os.path.dirname(expanded) or str(Path.home())
                prefix = os.path.basename(expanded)
            else:
                # Göreceli yol
                if '/' in partial:
                    rel_dir = os.path.dirname(partial)
                    target_dir = os.path.join(current_dir, rel_dir)
                    prefix = os.path.basename(partial)
                else:
                    target_dir = current_dir
                    prefix = partial
            
            if not os.path.exists(target_dir):
                return []
            
            matches = []
            for item in os.listdir(target_dir):
                if item.startswith(prefix):
                    item_path = os.path.join(target_dir, item)
                    
                    # Dizinlere / ekle
                    if os.path.isdir(item_path):
                        if partial.endswith('/') or not partial:
                            matches.append(item + '/')
                        else:
                            # Tam yol oluştur
                            if partial.startswith('/'):
                                matches.append(os.path.join(os.path.dirname(partial), item) + '/')
                            elif '/' in partial:
                                matches.append(os.path.join(os.path.dirname(partial), item) + '/')
                            else:
                                matches.append(item + '/')
                    else:
                        # Dosyalar
                        if partial.startswith('/'):
                            matches.append(os.path.join(os.path.dirname(partial), item))
                        elif '/' in partial:
                            matches.append(os.path.join(os.path.dirname(partial), item))
                        else:
                            matches.append(item)
            
            return sorted(matches)
            
        except Exception:
            return []
    
    def _complete_app_names(self) -> List[str]:
        """Uygulama adları tamamlama"""
        try:
            apps_dir = Path("../../apps")
            if not apps_dir.exists():
                apps_dir = Path("../../../apps")
            
            if not apps_dir.exists():
                return []
            
            app_names = []
            for app_dir in apps_dir.iterdir():
                if app_dir.is_dir() and (app_dir / "app.json").exists():
                    app_names.append(app_dir.name)
            
            return sorted(app_names)
            
        except Exception:
            return []
    
    def get_command_help(self, command: str) -> Optional[str]:
        """Komut yardımı al"""
        help_texts = {
            'help': "Show help for commands",
            'clear': "Clear the terminal screen",
            'exit': "Exit the terminal",
            'ls': "List directory contents",
            'cd': "Change directory",
            'pwd': "Print working directory",
            'cat': "Display file contents",
            'mkdir': "Create directories",
            'touch': "Create or update files",
            'ps': "Show running processes",
            'mem': "Show memory usage",
            'clapp': "Package manager for PyCloud apps - list, install, remove, update, search, info, doctor, help",
            'pycloud': "PyCloud system commands",
        }
        
        return help_texts.get(command.lower())
    
    def is_valid_command(self, command: str) -> bool:
        """Komutun geçerli olup olmadığını kontrol et"""
        return command.lower() in self.all_commands 


# Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
# For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt

"""Callback functions and support for sys.monitoring data collection."""

from __future__ import annotations

import functools
import inspect
import os
import os.path
import sys
import threading
import traceback

from dataclasses import dataclass
from types import CodeType
from typing import (
    Any,
    Callable,
    NewType,
    Optional,
    cast,
)

from coverage import env
from coverage.bytecode import TBranchTrails, branch_trails
from coverage.debug import short_filename, short_stack
from coverage.misc import isolate_module
from coverage.types import (
    AnyCallable,
    TFileDisposition,
    TLineNo,
    TOffset,
    TShouldStartContextFn,
    TShouldTraceFn,
    TTraceData,
    TTraceFileData,
    Tracer,
    TWarnFn,
)

os = isolate_module(os)

# pylint: disable=unused-argument

# $set_env.py: COVERAGE_SYSMON_LOG - Log sys.monitoring activity
LOG = bool(int(os.getenv("COVERAGE_SYSMON_LOG", 0)))

# $set_env.py: COVERAGE_SYSMON_STATS - Collect sys.monitoring stats
COLLECT_STATS = bool(int(os.getenv("COVERAGE_SYSMON_STATS", 0)))

# This module will be imported in all versions of Python, but only used in 3.12+
# It will be type-checked for 3.12, but not for earlier versions.
sys_monitoring = getattr(sys, "monitoring", None)

DISABLE_TYPE = NewType("DISABLE_TYPE", object)
MonitorReturn = Optional[DISABLE_TYPE]
DISABLE = cast(MonitorReturn, getattr(sys_monitoring, "DISABLE", None))


if LOG:  # pragma: debugging

    class LoggingWrapper:
        """Wrap a namespace to log all its functions."""

        def __init__(self, wrapped: Any, namespace: str) -> None:
            self.wrapped = wrapped
            self.namespace = namespace

        def __getattr__(self, name: str) -> Callable[..., Any]:
            def _wrapped(*args: Any, **kwargs: Any) -> Any:
                log(f"{self.namespace}.{name}{args}{kwargs}")
                return getattr(self.wrapped, name)(*args, **kwargs)

            return _wrapped

    sys_monitoring = LoggingWrapper(sys_monitoring, "sys.monitoring")
    assert sys_monitoring is not None

    short_stack = functools.partial(
        short_stack,
        full=True,
        short_filenames=True,
        frame_ids=True,
    )
    seen_threads: set[int] = set()

    def log(msg: str) -> None:
        """Write a message to our detailed debugging log(s)."""
        # Thread ids are reused across processes?
        # Make a shorter number more likely to be unique.
        pid = os.getpid()
        tid = cast(int, threading.current_thread().ident)
        tslug = f"{(pid * tid) % 9_999_991:07d}"
        if tid not in seen_threads:
            seen_threads.add(tid)
            log(f"New thread {tid} {tslug}:\n{short_stack()}")
        # log_seq = int(os.getenv("PANSEQ", "0"))
        # root = f"/tmp/pan.{log_seq:03d}"
        for filename in [
            "/tmp/foo.out",
            # f"{root}.out",
            # f"{root}-{pid}.out",
            # f"{root}-{pid}-{tslug}.out",
        ]:
            with open(filename, "a", encoding="utf-8") as f:
                try:
                    print(f"{pid}:{tslug}: {msg}", file=f, flush=True)
                except UnicodeError:
                    print(f"{pid}:{tslug}: {ascii(msg)}", file=f, flush=True)

    def arg_repr(arg: Any) -> str:
        """Make a customized repr for logged values."""
        if isinstance(arg, CodeType):
            return (
                f"<code @{id(arg):#x}"
                + f" name={arg.co_name},"
                + f" file={short_filename(arg.co_filename)!r}#{arg.co_firstlineno}>"
            )
        return repr(arg)

    def panopticon(*names: str | None) -> AnyCallable:
        """Decorate a function to log its calls."""

        def _decorator(method: AnyCallable) -> AnyCallable:
            @functools.wraps(method)
            def _wrapped(self: Any, *args: Any) -> Any:
                try:
                    # log(f"{method.__name__}() stack:\n{short_stack()}")
                    args_reprs = []
                    for name, arg in zip(names, args):
                        if name is None:
                            continue
                        args_reprs.append(f"{name}={arg_repr(arg)}")
                    log(f"{id(self):#x}:{method.__name__}({', '.join(args_reprs)})")
                    ret = method(self, *args)
                    # log(f" end {id(self):#x}:{method.__name__}({', '.join(args_reprs)})")
                    return ret
                except Exception as exc:
                    log(f"!!{exc.__class__.__name__}: {exc}")
                    if 1:
                        log("".join(traceback.format_exception(exc)))
                    try:
                        assert sys_monitoring is not None
                        sys_monitoring.set_events(sys.monitoring.COVERAGE_ID, 0)
                    except ValueError:
                        # We might have already shut off monitoring.
                        log("oops, shutting off events with disabled tool id")
                    raise

            return _wrapped

        return _decorator

else:

    def log(msg: str) -> None:
        """Write a message to our detailed debugging log(s), but not really."""

    def panopticon(*names: str | None) -> AnyCallable:
        """Decorate a function to log its calls, but not really."""

        def _decorator(meth: AnyCallable) -> AnyCallable:
            return meth

        return _decorator


@dataclass
class CodeInfo:
    """The information we want about each code object."""

    tracing: bool
    file_data: TTraceFileData | None
    byte_to_line: dict[TOffset, TLineNo] | None

    # Keys are start instruction offsets for branches.
    # Values are lists:
    #   [
    #       ([offset, offset, ...], (from_line, to_line)),
    #       ([offset, offset, ...], (from_line, to_line)),
    #   ]
    #   Two possible trails from the branch point, left and right.
    branch_trails: TBranchTrails


def bytes_to_lines(code: CodeType) -> dict[TOffset, TLineNo]:
    """Make a dict mapping byte code offsets to line numbers."""
    b2l = {}
    for bstart, bend, lineno in code.co_lines():
        if lineno is not None:
            for boffset in range(bstart, bend, 2):
                b2l[boffset] = lineno
    return b2l


class SysMonitor(Tracer):
    """Python implementation of the raw data tracer for PEP669 implementations."""

    # One of these will be used across threads. Be careful.

    def __init__(self, tool_id: int) -> None:
        # Attributes set from the collector:
        self.data: TTraceData
        self.trace_arcs = False
        self.should_trace: TShouldTraceFn
        self.should_trace_cache: dict[str, TFileDisposition | None]
        # TODO: should_start_context and switch_context are unused!
        # Change tests/testenv.py:DYN_CONTEXTS when this is updated.
        self.should_start_context: TShouldStartContextFn | None = None
        self.switch_context: Callable[[str | None], None] | None = None
        self.lock_data: Callable[[], None]
        self.unlock_data: Callable[[], None]
        # TODO: warn is unused.
        self.warn: TWarnFn

        self.myid = tool_id

        # Map id(code_object) -> CodeInfo
        self.code_infos: dict[int, CodeInfo] = {}
        # A list of code_objects, just to keep them alive so that id's are
        # useful as identity.
        self.code_objects: list[CodeType] = []
        self.sysmon_on = False
        self.lock = threading.Lock()

        self.stats: dict[str, int] | None = None
        if COLLECT_STATS:
            self.stats = dict.fromkeys(
                "starts start_tracing returns line_lines line_arcs branches branch_trails".split(),
                0,
            )

        self._activity = False

    def __repr__(self) -> str:
        points = sum(len(v) for v in self.data.values())
        files = len(self.data)
        return f"<SysMonitor at {id(self):#x}: {points} data points in {files} files>"

    @panopticon()
    def start(self) -> None:
        """Start this Tracer."""
        with self.lock:
            assert sys_monitoring is not None
            sys_monitoring.use_tool_id(self.myid, "coverage.py")
            register = functools.partial(sys_monitoring.register_callback, self.myid)
            events = sys.monitoring.events

            sys_monitoring.set_events(self.myid, events.PY_START)
            register(events.PY_START, self.sysmon_py_start)
            if self.trace_arcs:
                register(events.PY_RETURN, self.sysmon_py_return)
                register(events.LINE, self.sysmon_line_arcs)
                if env.PYBEHAVIOR.branch_right_left:
                    register(
                        events.BRANCH_RIGHT,  # type:ignore[attr-defined]
                        self.sysmon_branch_either,
                    )
                    register(
                        events.BRANCH_LEFT,  # type:ignore[attr-defined]
                        self.sysmon_branch_either,
                    )
            else:
                register(events.LINE, self.sysmon_line_lines)
            sys_monitoring.restart_events()
            self.sysmon_on = True

    @panopticon()
    def stop(self) -> None:
        """Stop this Tracer."""
        with self.lock:
            if not self.sysmon_on:
                # In forking situations, we might try to stop when we are not
                # started.  Do nothing in that case.
                return
            assert sys_monitoring is not None
            sys_monitoring.set_events(self.myid, 0)
            self.sysmon_on = False
            sys_monitoring.free_tool_id(self.myid)

    @panopticon()
    def post_fork(self) -> None:
        """The process has forked, clean up as needed."""
        self.stop()

    def activity(self) -> bool:
        """Has there been any activity?"""
        return self._activity

    def reset_activity(self) -> None:
        """Reset the activity() flag."""
        self._activity = False

    def get_stats(self) -> dict[str, int] | None:
        """Return a dictionary of statistics, or None."""
        return self.stats

    @panopticon("code", "@")
    def sysmon_py_start(
        self, code: CodeType, instruction_offset: TOffset
    ) -> MonitorReturn:
        """Handle sys.monitoring.events.PY_START events."""
        # Entering a new frame.  Decide if we should trace in this file.
        self._activity = True
        if self.stats is not None:
            self.stats["starts"] += 1

        code_info = self.code_infos.get(id(code))
        tracing_code: bool | None = None
        file_data: TTraceFileData | None = None
        if code_info is not None:
            tracing_code = code_info.tracing
            file_data = code_info.file_data

        if tracing_code is None:
            filename = code.co_filename
            disp = self.should_trace_cache.get(filename)
            if disp is None:
                frame = inspect.currentframe()
                if frame is not None:
                    frame = inspect.currentframe().f_back  # type: ignore[union-attr]
                    if LOG:
                        # @panopticon adds a frame.
                        frame = frame.f_back  # type: ignore[union-attr]
                disp = self.should_trace(filename, frame)  # type: ignore[arg-type]
                self.should_trace_cache[filename] = disp

            tracing_code = disp.trace
            if tracing_code:
                tracename = disp.source_filename
                assert tracename is not None
                self.lock_data()
                try:
                    if tracename not in self.data:
                        self.data[tracename] = set()
                finally:
                    self.unlock_data()
                file_data = self.data[tracename]
                b2l = bytes_to_lines(code)
            else:
                file_data = None
                b2l = None

            code_info = CodeInfo(
                tracing=tracing_code,
                file_data=file_data,
                byte_to_line=b2l,
                branch_trails={},
            )
            self.code_infos[id(code)] = code_info
            self.code_objects.append(code)

            if tracing_code:
                if self.stats is not None:
                    self.stats["start_tracing"] += 1
                events = sys.monitoring.events
                with self.lock:
                    if self.sysmon_on:
                        assert sys_monitoring is not None
                        local_events = events.PY_RETURN | events.PY_RESUME | events.LINE
                        if self.trace_arcs:
                            assert env.PYBEHAVIOR.branch_right_left
                            local_events |= (
                                events.BRANCH_RIGHT  # type:ignore[attr-defined]
                                | events.BRANCH_LEFT  # type:ignore[attr-defined]
                            )
                        sys_monitoring.set_local_events(self.myid, code, local_events)

        return DISABLE

    @panopticon("code", "@", None)
    def sysmon_py_return(
        self,
        code: CodeType,
        instruction_offset: TOffset,
        retval: object,
    ) -> MonitorReturn:
        """Handle sys.monitoring.events.PY_RETURN events for branch coverage."""
        if self.stats is not None:
            self.stats["returns"] += 1
        code_info = self.code_infos.get(id(code))
        # code_info is not None and code_info.file_data is not None, since we
        # wouldn't have enabled this event if they were.
        last_line = code_info.byte_to_line[instruction_offset]  # type: ignore
        if last_line is not None:
            arc = (last_line, -code.co_firstlineno)
            code_info.file_data.add(arc)  # type: ignore
            # log(f"adding {arc=}")
        return DISABLE

    @panopticon("code", "line")
    def sysmon_line_lines(self, code: CodeType, line_number: TLineNo) -> MonitorReturn:
        """Handle sys.monitoring.events.LINE events for line coverage."""
        if self.stats is not None:
            self.stats["line_lines"] += 1
        code_info = self.code_infos.get(id(code))
        # It should be true that code_info is not None and code_info.file_data
        # is not None, since we wouldn't have enabled this event if they were.
        # But somehow code_info can be None here, so we have to check.
        if code_info is not None and code_info.file_data is not None:
            code_info.file_data.add(line_number)  # type: ignore
        # log(f"adding {line_number=}")
        return DISABLE

    @panopticon("code", "line")
    def sysmon_line_arcs(self, code: CodeType, line_number: TLineNo) -> MonitorReturn:
        """Handle sys.monitoring.events.LINE events for branch coverage."""
        if self.stats is not None:
            self.stats["line_arcs"] += 1
        code_info = self.code_infos[id(code)]
        # code_info is not None and code_info.file_data is not None, since we
        # wouldn't have enabled this event if they were.
        arc = (line_number, line_number)
        code_info.file_data.add(arc)  # type: ignore
        # log(f"adding {arc=}")
        return DISABLE

    @panopticon("code", "@", "@")
    def sysmon_branch_either(
        self, code: CodeType, instruction_offset: TOffset, destination_offset: TOffset
    ) -> MonitorReturn:
        """Handle BRANCH_RIGHT and BRANCH_LEFT events."""
        if self.stats is not None:
            self.stats["branches"] += 1
        code_info = self.code_infos[id(code)]
        # code_info is not None and code_info.file_data is not None, since we
        # wouldn't have enabled this event if they were.
        if not code_info.branch_trails:
            if self.stats is not None:
                self.stats["branch_trails"] += 1
            code_info.branch_trails = branch_trails(code)
            # log(f"branch_trails for {code}:\n    {code_info.branch_trails}")
        added_arc = False
        dest_info = code_info.branch_trails.get(instruction_offset)
        # log(f"{dest_info = }")
        if dest_info is not None:
            for offsets, arc in dest_info:
                if arc is None:
                    continue
                if destination_offset in offsets:
                    code_info.file_data.add(arc)  # type: ignore
                    # log(f"adding {arc=}")
                    added_arc = True
                    break

        if not added_arc:
            # This could be an exception jumping from line to line.
            assert code_info.byte_to_line is not None
            l1 = code_info.byte_to_line[instruction_offset]
            l2 = code_info.byte_to_line.get(destination_offset)
            if l2 is not None and l1 != l2:
                arc = (l1, l2)
                code_info.file_data.add(arc)  # type: ignore
                # log(f"adding unforeseen {arc=}")

        return DISABLE



from __future__ import annotations

from collections.abc import Mapping
from datetime import date, datetime, time
from types import MappingProxyType

TYPE_CHECKING = False
if TYPE_CHECKING:
    from collections.abc import Generator
    from decimal import Decimal
    from typing import IO, Any, Final

ASCII_CTRL = frozenset(chr(i) for i in range(32)) | frozenset(chr(127))
ILLEGAL_BASIC_STR_CHARS = frozenset('"\\') | ASCII_CTRL - frozenset("\t")
BARE_KEY_CHARS = frozenset(
    "abcdefghijklmnopqrstuvwxyz" "ABCDEFGHIJKLMNOPQRSTUVWXYZ" "0123456789" "-_"
)
ARRAY_TYPES = (list, tuple)
MAX_LINE_LENGTH = 100

COMPACT_ESCAPES = MappingProxyType(
    {
        "\u0008": "\\b",  # backspace
        "\u000A": "\\n",  # linefeed
        "\u000C": "\\f",  # form feed
        "\u000D": "\\r",  # carriage return
        "\u0022": '\\"',  # quote
        "\u005C": "\\\\",  # backslash
    }
)


class Context:
    def __init__(self, allow_multiline: bool, indent: int):
        if indent < 0:
            raise ValueError("Indent width must be non-negative")
        self.allow_multiline: Final = allow_multiline
        # cache rendered inline tables (mapping from object id to rendered inline table)
        self.inline_table_cache: Final[dict[int, str]] = {}
        self.indent_str: Final = " " * indent


def dump(
    obj: Mapping[str, Any],
    fp: IO[bytes],
    /,
    *,
    multiline_strings: bool = False,
    indent: int = 4,
) -> None:
    ctx = Context(multiline_strings, indent)
    for chunk in gen_table_chunks(obj, ctx, name=""):
        fp.write(chunk.encode())


def dumps(
    obj: Mapping[str, Any], /, *, multiline_strings: bool = False, indent: int = 4
) -> str:
    ctx = Context(multiline_strings, indent)
    return "".join(gen_table_chunks(obj, ctx, name=""))


def gen_table_chunks(
    table: Mapping[str, Any],
    ctx: Context,
    *,
    name: str,
    inside_aot: bool = False,
) -> Generator[str, None, None]:
    yielded = False
    literals = []
    tables: list[tuple[str, Any, bool]] = []  # => [(key, value, inside_aot)]
    for k, v in table.items():
        if isinstance(v, Mapping):
            tables.append((k, v, False))
        elif is_aot(v) and not all(is_suitable_inline_table(t, ctx) for t in v):
            tables.extend((k, t, True) for t in v)
        else:
            literals.append((k, v))

    if inside_aot or name and (literals or not tables):
        yielded = True
        yield f"[[{name}]]\n" if inside_aot else f"[{name}]\n"

    if literals:
        yielded = True
        for k, v in literals:
            yield f"{format_key_part(k)} = {format_literal(v, ctx)}\n"

    for k, v, in_aot in tables:
        if yielded:
            yield "\n"
        else:
            yielded = True
        key_part = format_key_part(k)
        display_name = f"{name}.{key_part}" if name else key_part
        yield from gen_table_chunks(v, ctx, name=display_name, inside_aot=in_aot)


def format_literal(obj: object, ctx: Context, *, nest_level: int = 0) -> str:
    if isinstance(obj, bool):
        return "true" if obj else "false"
    if isinstance(obj, (int, float, date, datetime)):
        return str(obj)
    if isinstance(obj, time):
        if obj.tzinfo:
            raise ValueError("TOML does not support offset times")
        return str(obj)
    if isinstance(obj, str):
        return format_string(obj, allow_multiline=ctx.allow_multiline)
    if isinstance(obj, ARRAY_TYPES):
        return format_inline_array(obj, ctx, nest_level)
    if isinstance(obj, Mapping):
        return format_inline_table(obj, ctx)

    # Lazy import to improve module import time
    from decimal import Decimal

    if isinstance(obj, Decimal):
        return format_decimal(obj)
    raise TypeError(
        f"Object of type '{type(obj).__qualname__}' is not TOML serializable"
    )


def format_decimal(obj: Decimal) -> str:
    if obj.is_nan():
        return "nan"
    if obj.is_infinite():
        return "-inf" if obj.is_signed() else "inf"
    dec_str = str(obj).lower()
    return dec_str if "." in dec_str or "e" in dec_str else dec_str + ".0"


def format_inline_table(obj: Mapping, ctx: Context) -> str:
    # check cache first
    obj_id = id(obj)
    if obj_id in ctx.inline_table_cache:
        return ctx.inline_table_cache[obj_id]

    if not obj:
        rendered = "{}"
    else:
        rendered = (
            "{ "
            + ", ".join(
                f"{format_key_part(k)} = {format_literal(v, ctx)}"
                for k, v in obj.items()
            )
            + " }"
        )
    ctx.inline_table_cache[obj_id] = rendered
    return rendered


def format_inline_array(obj: tuple | list, ctx: Context, nest_level: int) -> str:
    if not obj:
        return "[]"
    item_indent = ctx.indent_str * (1 + nest_level)
    closing_bracket_indent = ctx.indent_str * nest_level
    return (
        "[\n"
        + ",\n".join(
            item_indent + format_literal(item, ctx, nest_level=nest_level + 1)
            for item in obj
        )
        + f",\n{closing_bracket_indent}]"
    )


def format_key_part(part: str) -> str:
    try:
        only_bare_key_chars = BARE_KEY_CHARS.issuperset(part)
    except TypeError:
        raise TypeError(
            f"Invalid mapping key '{part}' of type '{type(part).__qualname__}'."
            " A string is required."
        ) from None

    if part and only_bare_key_chars:
        return part
    return format_string(part, allow_multiline=False)


def format_string(s: str, *, allow_multiline: bool) -> str:
    do_multiline = allow_multiline and "\n" in s
    if do_multiline:
        result = '"""\n'
        s = s.replace("\r\n", "\n")
    else:
        result = '"'

    pos = seq_start = 0
    while True:
        try:
            char = s[pos]
        except IndexError:
            result += s[seq_start:pos]
            if do_multiline:
                return result + '"""'
            return result + '"'
        if char in ILLEGAL_BASIC_STR_CHARS:
            result += s[seq_start:pos]
            if char in COMPACT_ESCAPES:
                if do_multiline and char == "\n":
                    result += "\n"
                else:
                    result += COMPACT_ESCAPES[char]
            else:
                result += "\\u" + hex(ord(char))[2:].rjust(4, "0")
            seq_start = pos + 1
        pos += 1


def is_aot(obj: Any) -> bool:
    """Decides if an object behaves as an array of tables (i.e. a nonempty list
    of dicts)."""
    return bool(
        isinstance(obj, ARRAY_TYPES)
        and obj
        and all(isinstance(v, Mapping) for v in obj)
    )


def is_suitable_inline_table(obj: Mapping, ctx: Context) -> bool:
    """Use heuristics to decide if the inline-style representation is a good
    choice for a given table."""
    rendered_inline = f"{ctx.indent_str}{format_inline_table(obj, ctx)},"
    return len(rendered_inline) <= MAX_LINE_LENGTH and "\n" not in rendered_inline



"""Prepare for IR transform.

This needs to run after type checking and before generating IR.

For example, construct partially initialized FuncIR and ClassIR
objects for all functions and classes. This allows us to bind
references to functions and classes before we've generated full IR for
functions or classes.  The actual IR transform will then populate all
the missing bits, such as function bodies (basic blocks).

Also build a mapping from mypy TypeInfos to ClassIR objects.
"""

from __future__ import annotations

from collections import defaultdict
from collections.abc import Iterable
from typing import NamedTuple

from mypy.build import Graph
from mypy.nodes import (
    ARG_STAR,
    ARG_STAR2,
    CallExpr,
    ClassDef,
    Decorator,
    Expression,
    FuncDef,
    MemberExpr,
    MypyFile,
    NameExpr,
    OverloadedFuncDef,
    RefExpr,
    SymbolNode,
    TypeInfo,
    Var,
)
from mypy.semanal import refers_to_fullname
from mypy.traverser import TraverserVisitor
from mypy.types import Instance, Type, get_proper_type
from mypyc.common import PROPSET_PREFIX, get_id_from_name
from mypyc.crash import catch_errors
from mypyc.errors import Errors
from mypyc.ir.class_ir import ClassIR
from mypyc.ir.func_ir import (
    FUNC_CLASSMETHOD,
    FUNC_NORMAL,
    FUNC_STATICMETHOD,
    FuncDecl,
    FuncSignature,
    RuntimeArg,
)
from mypyc.ir.ops import DeserMaps
from mypyc.ir.rtypes import RInstance, RType, dict_rprimitive, none_rprimitive, tuple_rprimitive
from mypyc.irbuild.mapper import Mapper
from mypyc.irbuild.util import (
    get_func_def,
    get_mypyc_attrs,
    is_dataclass,
    is_extension_class,
    is_trait,
)
from mypyc.options import CompilerOptions
from mypyc.sametype import is_same_type


def build_type_map(
    mapper: Mapper,
    modules: list[MypyFile],
    graph: Graph,
    types: dict[Expression, Type],
    options: CompilerOptions,
    errors: Errors,
) -> None:
    # Collect all classes defined in everything we are compiling
    classes = []
    for module in modules:
        module_classes = [node for node in module.defs if isinstance(node, ClassDef)]
        classes.extend([(module, cdef) for cdef in module_classes])

    # Collect all class mappings so that we can bind arbitrary class name
    # references even if there are import cycles.
    for module, cdef in classes:
        class_ir = ClassIR(
            cdef.name,
            module.fullname,
            is_trait(cdef),
            is_abstract=cdef.info.is_abstract,
            is_final_class=cdef.info.is_final,
        )
        class_ir.is_ext_class = is_extension_class(module.path, cdef, errors)
        if class_ir.is_ext_class:
            class_ir.deletable = cdef.info.deletable_attributes.copy()
        # If global optimizations are disabled, turn of tracking of class children
        if not options.global_opts:
            class_ir.children = None
        mapper.type_to_ir[cdef.info] = class_ir
        mapper.symbol_fullnames.add(class_ir.fullname)

    # Populate structural information in class IR for extension classes.
    for module, cdef in classes:
        with catch_errors(module.path, cdef.line):
            if mapper.type_to_ir[cdef.info].is_ext_class:
                prepare_class_def(module.path, module.fullname, cdef, errors, mapper, options)
            else:
                prepare_non_ext_class_def(
                    module.path, module.fullname, cdef, errors, mapper, options
                )

    # Prepare implicit attribute accessors as needed if an attribute overrides a property.
    for module, cdef in classes:
        class_ir = mapper.type_to_ir[cdef.info]
        if class_ir.is_ext_class:
            prepare_implicit_property_accessors(cdef.info, class_ir, module.fullname, mapper)

    # Collect all the functions also. We collect from the symbol table
    # so that we can easily pick out the right copy of a function that
    # is conditionally defined.
    for module in modules:
        for func in get_module_func_defs(module):
            prepare_func_def(module.fullname, None, func, mapper, options)
            # TODO: what else?

    # Check for incompatible attribute definitions that were not
    # flagged by mypy but can't be supported when compiling.
    for module, cdef in classes:
        class_ir = mapper.type_to_ir[cdef.info]
        for attr in class_ir.attributes:
            for base_ir in class_ir.mro[1:]:
                if attr in base_ir.attributes:
                    if not is_same_type(class_ir.attributes[attr], base_ir.attributes[attr]):
                        node = cdef.info.names[attr].node
                        assert node is not None
                        kind = "trait" if base_ir.is_trait else "class"
                        errors.error(
                            f'Type of "{attr}" is incompatible with '
                            f'definition in {kind} "{base_ir.name}"',
                            module.path,
                            node.line,
                        )


def is_from_module(node: SymbolNode, module: MypyFile) -> bool:
    return node.fullname == module.fullname + "." + node.name


def load_type_map(mapper: Mapper, modules: list[MypyFile], deser_ctx: DeserMaps) -> None:
    """Populate a Mapper with deserialized IR from a list of modules."""
    for module in modules:
        for node in module.names.values():
            if isinstance(node.node, TypeInfo) and is_from_module(node.node, module):
                ir = deser_ctx.classes[node.node.fullname]
                mapper.type_to_ir[node.node] = ir
                mapper.symbol_fullnames.add(node.node.fullname)
                mapper.func_to_decl[node.node] = ir.ctor

    for module in modules:
        for func in get_module_func_defs(module):
            func_id = get_id_from_name(func.name, func.fullname, func.line)
            mapper.func_to_decl[func] = deser_ctx.functions[func_id].decl


def get_module_func_defs(module: MypyFile) -> Iterable[FuncDef]:
    """Collect all of the (non-method) functions declared in a module."""
    for node in module.names.values():
        # We need to filter out functions that are imported or
        # aliases.  The best way to do this seems to be by
        # checking that the fullname matches.
        if isinstance(node.node, (FuncDef, Decorator, OverloadedFuncDef)) and is_from_module(
            node.node, module
        ):
            yield get_func_def(node.node)


def prepare_func_def(
    module_name: str,
    class_name: str | None,
    fdef: FuncDef,
    mapper: Mapper,
    options: CompilerOptions,
) -> FuncDecl:
    kind = (
        FUNC_STATICMETHOD
        if fdef.is_static
        else (FUNC_CLASSMETHOD if fdef.is_class else FUNC_NORMAL)
    )
    sig = mapper.fdef_to_sig(fdef, options.strict_dunders_typing)
    decl = FuncDecl(fdef.name, class_name, module_name, sig, kind)
    mapper.func_to_decl[fdef] = decl
    return decl


def prepare_method_def(
    ir: ClassIR,
    module_name: str,
    cdef: ClassDef,
    mapper: Mapper,
    node: FuncDef | Decorator,
    options: CompilerOptions,
) -> None:
    if isinstance(node, FuncDef):
        ir.method_decls[node.name] = prepare_func_def(
            module_name, cdef.name, node, mapper, options
        )
    elif isinstance(node, Decorator):
        # TODO: do something about abstract methods here. Currently, they are handled just like
        # normal methods.
        decl = prepare_func_def(module_name, cdef.name, node.func, mapper, options)
        if not node.decorators:
            ir.method_decls[node.name] = decl
        elif isinstance(node.decorators[0], MemberExpr) and node.decorators[0].name == "setter":
            # Make property setter name different than getter name so there are no
            # name clashes when generating C code, and property lookup at the IR level
            # works correctly.
            decl.name = PROPSET_PREFIX + decl.name
            decl.is_prop_setter = True
            # Making the argument implicitly positional-only avoids unnecessary glue methods
            decl.sig.args[1].pos_only = True
            ir.method_decls[PROPSET_PREFIX + node.name] = decl

        if node.func.is_property:
            assert node.func.type, f"Expected return type annotation for property '{node.name}'"
            decl.is_prop_getter = True
            ir.property_types[node.name] = decl.sig.ret_type


def is_valid_multipart_property_def(prop: OverloadedFuncDef) -> bool:
    # Checks to ensure supported property decorator semantics
    if len(prop.items) != 2:
        return False

    getter = prop.items[0]
    setter = prop.items[1]

    return (
        isinstance(getter, Decorator)
        and isinstance(setter, Decorator)
        and getter.func.is_property
        and len(setter.decorators) == 1
        and isinstance(setter.decorators[0], MemberExpr)
        and setter.decorators[0].name == "setter"
    )


def can_subclass_builtin(builtin_base: str) -> bool:
    # BaseException and dict are special cased.
    return builtin_base in (
        (
            "builtins.Exception",
            "builtins.LookupError",
            "builtins.IndexError",
            "builtins.Warning",
            "builtins.UserWarning",
            "builtins.ValueError",
            "builtins.object",
        )
    )


def prepare_class_def(
    path: str,
    module_name: str,
    cdef: ClassDef,
    errors: Errors,
    mapper: Mapper,
    options: CompilerOptions,
) -> None:
    """Populate the interface-level information in a class IR.

    This includes attribute and method declarations, and the MRO, among other things, but
    method bodies are generated in a later pass.
    """

    ir = mapper.type_to_ir[cdef.info]
    info = cdef.info

    attrs = get_mypyc_attrs(cdef)
    if attrs.get("allow_interpreted_subclasses") is True:
        ir.allow_interpreted_subclasses = True
    if attrs.get("serializable") is True:
        # Supports copy.copy and pickle (including subclasses)
        ir._serializable = True

    # Check for subclassing from builtin types
    for cls in info.mro:
        # Special case exceptions and dicts
        # XXX: How do we handle *other* things??
        if cls.fullname == "builtins.BaseException":
            ir.builtin_base = "PyBaseExceptionObject"
        elif cls.fullname == "builtins.dict":
            ir.builtin_base = "PyDictObject"
        elif cls.fullname.startswith("builtins."):
            if not can_subclass_builtin(cls.fullname):
                # Note that if we try to subclass a C extension class that
                # isn't in builtins, bad things will happen and we won't
                # catch it here! But this should catch a lot of the most
                # common pitfalls.
                errors.error(
                    "Inheriting from most builtin types is unimplemented", path, cdef.line
                )
                errors.note(
                    "Potential workaround: @mypy_extensions.mypyc_attr(native_class=False)",
                    path,
                    cdef.line,
                )
                errors.note(
                    "https://mypyc.readthedocs.io/en/stable/native_classes.html#defining-non-native-classes",
                    path,
                    cdef.line,
                )

    # Set up the parent class
    bases = [mapper.type_to_ir[base.type] for base in info.bases if base.type in mapper.type_to_ir]
    if len(bases) > 1 and any(not c.is_trait for c in bases) and bases[0].is_trait:
        # If the first base is a non-trait, don't ever error here. While it is correct
        # to error if a trait comes before the next non-trait base (e.g. non-trait, trait,
        # non-trait), it's pointless, confusing noise from the bigger issue: multiple
        # inheritance is *not* supported.
        errors.error("Non-trait base must appear first in parent list", path, cdef.line)
    ir.traits = [c for c in bases if c.is_trait]

    mro = []  # All mypyc base classes
    base_mro = []  # Non-trait mypyc base classes
    for cls in info.mro:
        if cls not in mapper.type_to_ir:
            if cls.fullname != "builtins.object":
                ir.inherits_python = True
            continue
        base_ir = mapper.type_to_ir[cls]
        if not base_ir.is_trait:
            base_mro.append(base_ir)
        mro.append(base_ir)

        if cls.defn.removed_base_type_exprs or not base_ir.is_ext_class:
            ir.inherits_python = True

    base_idx = 1 if not ir.is_trait else 0
    if len(base_mro) > base_idx:
        ir.base = base_mro[base_idx]
    ir.mro = mro
    ir.base_mro = base_mro

    prepare_methods_and_attributes(cdef, ir, path, module_name, errors, mapper, options)
    prepare_init_method(cdef, ir, module_name, mapper)

    for base in bases:
        if base.children is not None:
            base.children.append(ir)

    if is_dataclass(cdef):
        ir.is_augmented = True


def prepare_methods_and_attributes(
    cdef: ClassDef,
    ir: ClassIR,
    path: str,
    module_name: str,
    errors: Errors,
    mapper: Mapper,
    options: CompilerOptions,
) -> None:
    """Populate attribute and method declarations."""
    info = cdef.info
    for name, node in info.names.items():
        # Currently all plugin generated methods are dummies and not included.
        if node.plugin_generated:
            continue

        if isinstance(node.node, Var):
            assert node.node.type, "Class member %s missing type" % name
            if not node.node.is_classvar and name not in ("__slots__", "__deletable__"):
                attr_rtype = mapper.type_to_rtype(node.node.type)
                if ir.is_trait and attr_rtype.error_overlap:
                    # Traits don't have attribute definedness bitmaps, so use
                    # property accessor methods to access attributes that need them.
                    # We will generate accessor implementations that use the class bitmap
                    # for any concrete subclasses.
                    add_getter_declaration(ir, name, attr_rtype, module_name)
                    add_setter_declaration(ir, name, attr_rtype, module_name)
                ir.attributes[name] = attr_rtype
        elif isinstance(node.node, (FuncDef, Decorator)):
            prepare_method_def(ir, module_name, cdef, mapper, node.node, options)
        elif isinstance(node.node, OverloadedFuncDef):
            # Handle case for property with both a getter and a setter
            if node.node.is_property:
                if is_valid_multipart_property_def(node.node):
                    for item in node.node.items:
                        prepare_method_def(ir, module_name, cdef, mapper, item, options)
                else:
                    errors.error("Unsupported property decorator semantics", path, cdef.line)

            # Handle case for regular function overload
            else:
                if not node.node.impl:
                    errors.error(
                        "Overloads without implementation are not supported", path, cdef.line
                    )
                else:
                    prepare_method_def(ir, module_name, cdef, mapper, node.node.impl, options)

    if ir.builtin_base:
        ir.attributes.clear()


def prepare_implicit_property_accessors(
    info: TypeInfo, ir: ClassIR, module_name: str, mapper: Mapper
) -> None:
    concrete_attributes = set()
    for base in ir.base_mro:
        for name, attr_rtype in base.attributes.items():
            concrete_attributes.add(name)
            add_property_methods_for_attribute_if_needed(
                info, ir, name, attr_rtype, module_name, mapper
            )
    for base in ir.mro[1:]:
        if base.is_trait:
            for name, attr_rtype in base.attributes.items():
                if name not in concrete_attributes:
                    add_property_methods_for_attribute_if_needed(
                        info, ir, name, attr_rtype, module_name, mapper
                    )


def add_property_methods_for_attribute_if_needed(
    info: TypeInfo,
    ir: ClassIR,
    attr_name: str,
    attr_rtype: RType,
    module_name: str,
    mapper: Mapper,
) -> None:
    """Add getter and/or setter for attribute if defined as property in a base class.

    Only add declarations. The body IR will be synthesized later during irbuild.
    """
    for base in info.mro[1:]:
        if base in mapper.type_to_ir:
            base_ir = mapper.type_to_ir[base]
            n = base.names.get(attr_name)
            if n is None:
                continue
            node = n.node
            if isinstance(node, Decorator) and node.name not in ir.method_decls:
                # Defined as a read-only property in base class/trait
                add_getter_declaration(ir, attr_name, attr_rtype, module_name)
            elif isinstance(node, OverloadedFuncDef) and is_valid_multipart_property_def(node):
                # Defined as a read-write property in base class/trait
                add_getter_declaration(ir, attr_name, attr_rtype, module_name)
                add_setter_declaration(ir, attr_name, attr_rtype, module_name)
            elif base_ir.is_trait and attr_rtype.error_overlap:
                add_getter_declaration(ir, attr_name, attr_rtype, module_name)
                add_setter_declaration(ir, attr_name, attr_rtype, module_name)


def add_getter_declaration(
    ir: ClassIR, attr_name: str, attr_rtype: RType, module_name: str
) -> None:
    self_arg = RuntimeArg("self", RInstance(ir), pos_only=True)
    sig = FuncSignature([self_arg], attr_rtype)
    decl = FuncDecl(attr_name, ir.name, module_name, sig, FUNC_NORMAL)
    decl.is_prop_getter = True
    decl.implicit = True  # Triggers synthesization
    ir.method_decls[attr_name] = decl
    ir.property_types[attr_name] = attr_rtype  # TODO: Needed??


def add_setter_declaration(
    ir: ClassIR, attr_name: str, attr_rtype: RType, module_name: str
) -> None:
    self_arg = RuntimeArg("self", RInstance(ir), pos_only=True)
    value_arg = RuntimeArg("value", attr_rtype, pos_only=True)
    sig = FuncSignature([self_arg, value_arg], none_rprimitive)
    setter_name = PROPSET_PREFIX + attr_name
    decl = FuncDecl(setter_name, ir.name, module_name, sig, FUNC_NORMAL)
    decl.is_prop_setter = True
    decl.implicit = True  # Triggers synthesization
    ir.method_decls[setter_name] = decl


def prepare_init_method(cdef: ClassDef, ir: ClassIR, module_name: str, mapper: Mapper) -> None:
    # Set up a constructor decl
    init_node = cdef.info["__init__"].node
    if not ir.is_trait and not ir.builtin_base and isinstance(init_node, FuncDef):
        init_sig = mapper.fdef_to_sig(init_node, True)

        defining_ir = mapper.type_to_ir.get(init_node.info)
        # If there is a nontrivial __init__ that wasn't defined in an
        # extension class, we need to make the constructor take *args,
        # **kwargs so it can call tp_init.
        if (
            defining_ir is None
            or not defining_ir.is_ext_class
            or cdef.info["__init__"].plugin_generated
        ) and init_node.info.fullname != "builtins.object":
            init_sig = FuncSignature(
                [
                    init_sig.args[0],
                    RuntimeArg("args", tuple_rprimitive, ARG_STAR),
                    RuntimeArg("kwargs", dict_rprimitive, ARG_STAR2),
                ],
                init_sig.ret_type,
            )

        last_arg = len(init_sig.args) - init_sig.num_bitmap_args
        ctor_sig = FuncSignature(init_sig.args[1:last_arg], RInstance(ir))
        ir.ctor = FuncDecl(cdef.name, None, module_name, ctor_sig)
        mapper.func_to_decl[cdef.info] = ir.ctor


def prepare_non_ext_class_def(
    path: str,
    module_name: str,
    cdef: ClassDef,
    errors: Errors,
    mapper: Mapper,
    options: CompilerOptions,
) -> None:
    ir = mapper.type_to_ir[cdef.info]
    info = cdef.info

    for node in info.names.values():
        if isinstance(node.node, (FuncDef, Decorator)):
            prepare_method_def(ir, module_name, cdef, mapper, node.node, options)
        elif isinstance(node.node, OverloadedFuncDef):
            # Handle case for property with both a getter and a setter
            if node.node.is_property:
                if not is_valid_multipart_property_def(node.node):
                    errors.error("Unsupported property decorator semantics", path, cdef.line)
                for item in node.node.items:
                    prepare_method_def(ir, module_name, cdef, mapper, item, options)
            # Handle case for regular function overload
            else:
                prepare_method_def(ir, module_name, cdef, mapper, get_func_def(node.node), options)

    if any(cls in mapper.type_to_ir and mapper.type_to_ir[cls].is_ext_class for cls in info.mro):
        errors.error(
            "Non-extension classes may not inherit from extension classes", path, cdef.line
        )


RegisterImplInfo = tuple[TypeInfo, FuncDef]


class SingledispatchInfo(NamedTuple):
    singledispatch_impls: dict[FuncDef, list[RegisterImplInfo]]
    decorators_to_remove: dict[FuncDef, list[int]]


def find_singledispatch_register_impls(
    modules: list[MypyFile], errors: Errors
) -> SingledispatchInfo:
    visitor = SingledispatchVisitor(errors)
    for module in modules:
        visitor.current_path = module.path
        module.accept(visitor)
    return SingledispatchInfo(visitor.singledispatch_impls, visitor.decorators_to_remove)


class SingledispatchVisitor(TraverserVisitor):
    current_path: str

    def __init__(self, errors: Errors) -> None:
        super().__init__()

        # Map of main singledispatch function to list of registered implementations
        self.singledispatch_impls: defaultdict[FuncDef, list[RegisterImplInfo]] = defaultdict(list)

        # Map of decorated function to the indices of any decorators to remove
        self.decorators_to_remove: dict[FuncDef, list[int]] = {}

        self.errors: Errors = errors

    def visit_decorator(self, dec: Decorator) -> None:
        if dec.decorators:
            decorators_to_store = dec.decorators.copy()
            decorators_to_remove: list[int] = []
            # the index of the last non-register decorator before finding a register decorator
            # when going through decorators from top to bottom
            last_non_register: int | None = None
            for i, d in enumerate(decorators_to_store):
                impl = get_singledispatch_register_call_info(d, dec.func)
                if impl is not None:
                    self.singledispatch_impls[impl.singledispatch_func].append(
                        (impl.dispatch_type, dec.func)
                    )
                    decorators_to_remove.append(i)
                    if last_non_register is not None:
                        # found a register decorator after a non-register decorator, which we
                        # don't support because we'd have to make a copy of the function before
                        # calling the decorator so that we can call it later, which complicates
                        # the implementation for something that is probably not commonly used
                        self.errors.error(
                            "Calling decorator after registering function not supported",
                            self.current_path,
                            decorators_to_store[last_non_register].line,
                        )
                else:
                    if refers_to_fullname(d, "functools.singledispatch"):
                        decorators_to_remove.append(i)
                        # make sure that we still treat the function as a singledispatch function
                        # even if we don't find any registered implementations (which might happen
                        # if all registered implementations are registered dynamically)
                        self.singledispatch_impls.setdefault(dec.func, [])
                    last_non_register = i

            if decorators_to_remove:
                # calling register on a function that tries to dispatch based on type annotations
                # raises a TypeError because compiled functions don't have an __annotations__
                # attribute
                self.decorators_to_remove[dec.func] = decorators_to_remove

        super().visit_decorator(dec)


class RegisteredImpl(NamedTuple):
    singledispatch_func: FuncDef
    dispatch_type: TypeInfo


def get_singledispatch_register_call_info(
    decorator: Expression, func: FuncDef
) -> RegisteredImpl | None:
    # @fun.register(complex)
    # def g(arg): ...
    if (
        isinstance(decorator, CallExpr)
        and len(decorator.args) == 1
        and isinstance(decorator.args[0], RefExpr)
    ):
        callee = decorator.callee
        dispatch_type = decorator.args[0].node
        if not isinstance(dispatch_type, TypeInfo):
            return None

        if isinstance(callee, MemberExpr):
            return registered_impl_from_possible_register_call(callee, dispatch_type)
    # @fun.register
    # def g(arg: int): ...
    elif isinstance(decorator, MemberExpr):
        # we don't know if this is a register call yet, so we can't be sure that the function
        # actually has arguments
        if not func.arguments:
            return None
        arg_type = get_proper_type(func.arguments[0].variable.type)
        if not isinstance(arg_type, Instance):
            return None
        info = arg_type.type
        return registered_impl_from_possible_register_call(decorator, info)
    return None


def registered_impl_from_possible_register_call(
    expr: MemberExpr, dispatch_type: TypeInfo
) -> RegisteredImpl | None:
    if expr.name == "register" and isinstance(expr.expr, NameExpr):
        node = expr.expr.node
        if isinstance(node, Decorator):
            return RegisteredImpl(node.func, dispatch_type)
    return None



"""
PyCloud OS Thread Manager
Tüm sistem ve uygulamalar için güvenli, izlenebilir ve esnek thread yürütme altyapısı
"""

import os
import time
import uuid
import pickle
import logging
import threading
import queue
import weakref
from pathlib import Path
from typing import Dict, List, Optional, Callable, Any, Union
from datetime import datetime
from dataclasses import dataclass, asdict, field
from enum import Enum
from concurrent.futures import ThreadPoolExecutor, Future

class ThreadStatus(Enum):
    """Thread durumları"""
    IDLE = "idle"
    RUNNING = "running"
    FINISHED = "finished"
    CRASHED = "crashed"
    FROZEN = "frozen"
    CANCELLED = "cancelled"

class ThreadPriority(Enum):
    """Thread öncelikleri"""
    LOW = 1
    NORMAL = 2
    HIGH = 3
    CRITICAL = 4

class ThreadType(Enum):
    """Thread türleri"""
    SYSTEM = "system"
    APPLICATION = "application"
    SERVICE = "service"
    USER = "user"
    BACKGROUND = "background"

@dataclass
class ThreadInfo:
    """Thread bilgi sınıfı"""
    tid: str  # Thread ID
    name: str
    thread_type: ThreadType
    priority: ThreadPriority
    status: ThreadStatus
    app_id: Optional[str] = None
    created_at: str = field(default_factory=lambda: datetime.now().isoformat())
    started_at: Optional[str] = None
    finished_at: Optional[str] = None
    runtime_seconds: float = 0.0
    error_message: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    
    def to_dict(self) -> Dict:
        """Dict'e çevir"""
        data = asdict(self)
        data['thread_type'] = self.thread_type.value
        data['priority'] = self.priority.value
        data['status'] = self.status.value
        return data

@dataclass
class ThreadProfile:
    """Thread profil bilgisi"""
    tid: str
    execution_time: float
    memory_usage_mb: float
    cpu_usage_percent: float
    function_calls: int
    io_operations: int
    created_objects: int
    peak_memory_mb: float
    
    def to_dict(self) -> Dict:
        """Dict'e çevir"""
        return asdict(self)

@dataclass
class ThreadSnapshot:
    """Thread snapshot bilgisi"""
    tid: str
    timestamp: str
    status: ThreadStatus
    local_vars: Dict[str, Any]
    execution_point: str
    stack_trace: List[str]
    memory_state: bytes
    
    def to_dict(self) -> Dict:
        """Dict'e çevir (memory_state hariç)"""
        data = asdict(self)
        data['status'] = self.status.value
        data.pop('memory_state', None)  # Binary veri çıkar
        return data

@dataclass
class ThreadMessage:
    """Thread mesaj sınıfı"""
    message_id: str
    from_tid: str
    to_tid: str
    message_type: str
    payload: Any
    timestamp: str = field(default_factory=lambda: datetime.now().isoformat())
    delivered: bool = False

class ThreadQueue:
    """Thread kuyruğu yöneticisi"""
    
    def __init__(self, max_size: int = 1000):
        self.queue = queue.PriorityQueue(maxsize=max_size)
        self.logger = logging.getLogger("ThreadQueue")
        self._lock = threading.Lock()
        self.stats = {
            "tasks_queued": 0,
            "tasks_executed": 0,
            "tasks_failed": 0
        }
    
    def enqueue(self, func: Callable, args: tuple = (), kwargs: dict = None,
                priority: ThreadPriority = ThreadPriority.NORMAL,
                thread_info: ThreadInfo = None) -> str:
        """Görevi kuyruğa ekle"""
        if kwargs is None:
            kwargs = {}
        
        task_id = str(uuid.uuid4())
        
        # Öncelik değerini ters çevir (düşük sayı = yüksek öncelik)
        priority_value = 5 - priority.value
        
        task = {
            "id": task_id,
            "func": func,
            "args": args,
            "kwargs": kwargs,
            "thread_info": thread_info,
            "queued_at": time.time()
        }
        
        try:
            self.queue.put((priority_value, task), block=False)
            self.stats["tasks_queued"] += 1
            self.logger.debug(f"Task queued: {task_id}")
            return task_id
        except queue.Full:
            self.logger.error("Thread queue is full")
            return None
    
    def dequeue(self, timeout: float = 1.0) -> Optional[Dict]:
        """Kuyruktan görev al"""
        try:
            priority, task = self.queue.get(timeout=timeout)
            return task
        except queue.Empty:
            return None
    
    def get_stats(self) -> Dict:
        """Kuyruk istatistikleri"""
        return {
            **self.stats,
            "queue_size": self.queue.qsize(),
            "queue_full": self.queue.full()
        }

class ThreadPool:
    """Thread havuzu yöneticisi"""
    
    def __init__(self, max_workers: int = None):
        if max_workers is None:
            max_workers = min(32, (os.cpu_count() or 1) + 4)
        
        self.max_workers = max_workers
        self.executor = ThreadPoolExecutor(max_workers=max_workers)
        self.logger = logging.getLogger("ThreadPool")
        self.active_futures: Dict[str, Future] = {}
        self.completed_tasks = 0
        self.failed_tasks = 0
        
    def submit(self, func: Callable, *args, **kwargs) -> str:
        """Görevi havuza gönder"""
        task_id = str(uuid.uuid4())
        
        try:
            future = self.executor.submit(func, *args, **kwargs)
            self.active_futures[task_id] = future
            
            # Tamamlanma callback'i
            future.add_done_callback(lambda f: self._on_task_completed(task_id, f))
            
            self.logger.debug(f"Task submitted to pool: {task_id}")
            return task_id
            
        except Exception as e:
            self.logger.error(f"Failed to submit task: {e}")
            return None
    
    def _on_task_completed(self, task_id: str, future: Future):
        """Görev tamamlanma callback'i"""
        try:
            if future.exception():
                self.failed_tasks += 1
                self.logger.error(f"Task {task_id} failed: {future.exception()}")
            else:
                self.completed_tasks += 1
                self.logger.debug(f"Task {task_id} completed successfully")
        except Exception as e:
            self.logger.error(f"Error in task completion callback: {e}")
        finally:
            # Future'ı temizle
            self.active_futures.pop(task_id, None)
    
    def cancel_task(self, task_id: str) -> bool:
        """Görevi iptal et"""
        future = self.active_futures.get(task_id)
        if future:
            return future.cancel()
        return False
    
    def get_stats(self) -> Dict:
        """Havuz istatistikleri"""
        return {
            "max_workers": self.max_workers,
            "active_tasks": len(self.active_futures),
            "completed_tasks": self.completed_tasks,
            "failed_tasks": self.failed_tasks
        }
    
    def shutdown(self, wait: bool = True):
        """Havuzu kapat"""
        self.executor.shutdown(wait=wait)

class ThreadMessaging:
    """Thread'ler arası mesajlaşma sistemi"""
    
    def __init__(self):
        self.logger = logging.getLogger("ThreadMessaging")
        self.message_queues: Dict[str, queue.Queue] = {}
        self.message_history: List[ThreadMessage] = []
        self.max_history = 1000
        self._lock = threading.Lock()
        
        # Mesaj filtreleri
        self.message_filters: Dict[str, Callable] = {}
    
    def register_thread(self, tid: str):
        """Thread'i mesajlaşma için kaydet"""
        with self._lock:
            if tid not in self.message_queues:
                self.message_queues[tid] = queue.Queue(maxsize=100)
                self.logger.debug(f"Thread registered for messaging: {tid}")
    
    def unregister_thread(self, tid: str):
        """Thread'i mesajlaşmadan çıkar"""
        with self._lock:
            if tid in self.message_queues:
                del self.message_queues[tid]
                self.logger.debug(f"Thread unregistered from messaging: {tid}")
    
    def send_message(self, from_tid: str, to_tid: str, message_type: str, payload: Any) -> bool:
        """Mesaj gönder"""
        try:
            # Hedef thread kontrolü
            if to_tid not in self.message_queues:
                self.logger.warning(f"Target thread {to_tid} not registered")
                return False
            
            # Mesaj oluştur
            message = ThreadMessage(
                message_id=str(uuid.uuid4()),
                from_tid=from_tid,
                to_tid=to_tid,
                message_type=message_type,
                payload=payload
            )
            
            # Filtre kontrolü
            if not self._check_message_filter(message):
                self.logger.warning(f"Message filtered: {message_type}")
                return False
            
            # Mesajı kuyruğa ekle
            target_queue = self.message_queues[to_tid]
            target_queue.put(message, block=False)
            
            # Geçmişe ekle
            self._add_to_history(message)
            
            self.logger.debug(f"Message sent: {from_tid} -> {to_tid} ({message_type})")
            return True
            
        except queue.Full:
            self.logger.error(f"Message queue full for thread {to_tid}")
            return False
        except Exception as e:
            self.logger.error(f"Failed to send message: {e}")
            return False
    
    def receive_message(self, tid: str, timeout: float = 0.1) -> Optional[ThreadMessage]:
        """Mesaj al"""
        try:
            if tid not in self.message_queues:
                return None
            
            message_queue = self.message_queues[tid]
            message = message_queue.get(timeout=timeout)
            message.delivered = True
            
            return message
            
        except queue.Empty:
            return None
        except Exception as e:
            self.logger.error(f"Failed to receive message: {e}")
            return None
    
    def _check_message_filter(self, message: ThreadMessage) -> bool:
        """Mesaj filtresi kontrolü"""
        filter_func = self.message_filters.get(message.message_type)
        if filter_func:
            try:
                return filter_func(message)
            except Exception as e:
                self.logger.error(f"Message filter error: {e}")
                return False
        return True
    
    def _add_to_history(self, message: ThreadMessage):
        """Mesajı geçmişe ekle"""
        self.message_history.append(message)
        if len(self.message_history) > self.max_history:
            self.message_history = self.message_history[-self.max_history:]
    
    def add_message_filter(self, message_type: str, filter_func: Callable):
        """Mesaj filtresi ekle"""
        self.message_filters[message_type] = filter_func
    
    def get_message_history(self, tid: str = None, limit: int = 100) -> List[Dict]:
        """Mesaj geçmişi al"""
        history = self.message_history
        
        if tid:
            history = [m for m in history if m.from_tid == tid or m.to_tid == tid]
        
        return [m.__dict__ for m in history[-limit:]]

class ThreadSnapshot:
    """Thread snapshot yöneticisi"""
    
    def __init__(self):
        self.logger = logging.getLogger("ThreadSnapshot")
        self.snapshots: Dict[str, ThreadSnapshot] = {}
        self.snapshots_dir = Path("temp/thread_snapshots")
        self.snapshots_dir.mkdir(parents=True, exist_ok=True)
    
    def create_snapshot(self, tid: str, thread_info: ThreadInfo) -> bool:
        """Thread snapshot oluştur"""
        try:
            # Stack trace al
            import traceback
            stack_trace = traceback.format_stack()
            
            # Local variables (basit implementasyon)
            local_vars = {}
            
            # Memory state (placeholder)
            memory_state = b""
            
            snapshot = ThreadSnapshot(
                tid=tid,
                timestamp=datetime.now().isoformat(),
                status=thread_info.status,
                local_vars=local_vars,
                execution_point=f"Line {len(stack_trace)}",
                stack_trace=stack_trace,
                memory_state=memory_state
            )
            
            # Dosyaya kaydet
            snapshot_file = self.snapshots_dir / f"{tid}_{int(time.time())}.snapshot"
            with open(snapshot_file, 'wb') as f:
                pickle.dump(snapshot, f)
            
            self.snapshots[tid] = snapshot
            self.logger.info(f"Snapshot created for thread {tid}")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to create snapshot for {tid}: {e}")
            return False
    
    def restore_snapshot(self, tid: str) -> Optional[ThreadSnapshot]:
        """Snapshot'ı geri yükle"""
        try:
            if tid in self.snapshots:
                return self.snapshots[tid]
            
            # Dosyadan yükle
            snapshot_files = list(self.snapshots_dir.glob(f"{tid}_*.snapshot"))
            if snapshot_files:
                latest_file = max(snapshot_files, key=lambda f: f.stat().st_mtime)
                
                with open(latest_file, 'rb') as f:
                    snapshot = pickle.load(f)
                
                self.snapshots[tid] = snapshot
                return snapshot
            
            return None
            
        except Exception as e:
            self.logger.error(f"Failed to restore snapshot for {tid}: {e}")
            return None

class ThreadProfiler:
    """Thread profiling sistemi"""
    
    def __init__(self):
        self.logger = logging.getLogger("ThreadProfiler")
        self.profiles: Dict[str, ThreadProfile] = {}
        self.profiling_enabled = True
    
    def start_profiling(self, tid: str):
        """Profiling başlat"""
        if not self.profiling_enabled:
            return
        
        try:
            profile = ThreadProfile(
                tid=tid,
                execution_time=0.0,
                memory_usage_mb=0.0,
                cpu_usage_percent=0.0,
                function_calls=0,
                io_operations=0,
                created_objects=0,
                peak_memory_mb=0.0
            )
            
            self.profiles[tid] = profile
            self.logger.debug(f"Profiling started for thread {tid}")
            
        except Exception as e:
            self.logger.error(f"Failed to start profiling for {tid}: {e}")
    
    def update_profile(self, tid: str, **metrics):
        """Profil güncelle"""
        if tid in self.profiles:
            profile = self.profiles[tid]
            
            for key, value in metrics.items():
                if hasattr(profile, key):
                    setattr(profile, key, value)
    
    def finish_profiling(self, tid: str) -> Optional[ThreadProfile]:
        """Profiling bitir"""
        return self.profiles.pop(tid, None)
    
    def get_profile(self, tid: str) -> Optional[ThreadProfile]:
        """Profil al"""
        return self.profiles.get(tid)

class ThreadManager:
    """Ana thread yöneticisi"""
    
    def __init__(self, kernel=None):
        self.kernel = kernel
        self.logger = logging.getLogger("ThreadManager")
        
        # Thread bilgileri
        self.threads: Dict[str, ThreadInfo] = {}
        self.thread_objects: Dict[str, threading.Thread] = weakref.WeakValueDictionary()
        
        # Alt sistemler
        self.queue = ThreadQueue()
        self.pool = ThreadPool()
        self.messaging = ThreadMessaging()
        self.snapshots = ThreadSnapshot()
        self.profiler = ThreadProfiler()
        
        # Watchdog
        self.watchdog_running = False
        self.watchdog_thread = None
        self.watchdog_interval = 5.0
        
        # Callbacks
        self.callbacks: Dict[str, List[Callable]] = {
            "thread_started": [],
            "thread_finished": [],
            "thread_crashed": [],
            "thread_frozen": []
        }
        
        self.start_watchdog()
    
    def create_thread(self, func: Callable, args: tuple = (), kwargs: dict = None,
                     name: str = None, app_id: str = None,
                     thread_type: ThreadType = ThreadType.APPLICATION,
                     priority: ThreadPriority = ThreadPriority.NORMAL,
                     enable_profiling: bool = True,
                     enable_messaging: bool = True) -> str:
        """Yeni thread oluştur"""
        
        if kwargs is None:
            kwargs = {}
        
        # Thread ID oluştur
        tid = str(uuid.uuid4())
        
        if not name:
            name = f"Thread-{tid[:8]}"
        
        # Thread bilgisi oluştur
        thread_info = ThreadInfo(
            tid=tid,
            name=name,
            thread_type=thread_type,
            priority=priority,
            status=ThreadStatus.IDLE,
            app_id=app_id
        )
        
        # Kaydet
        self.threads[tid] = thread_info
        
        # Mesajlaşma kaydı
        if enable_messaging:
            self.messaging.register_thread(tid)
        
        # Profiling başlat
        if enable_profiling:
            self.profiler.start_profiling(tid)
        
        self.logger.info(f"Thread created: {name} ({tid})")
        return tid
    
    def start_thread(self, tid: str, func: Callable, args: tuple = (), kwargs: dict = None) -> bool:
        """Thread'i başlat"""
        if tid not in self.threads:
            self.logger.error(f"Thread {tid} not found")
            return False
        
        if kwargs is None:
            kwargs = {}
        
        thread_info = self.threads[tid]
        
        try:
            # Wrapper fonksiyon
            def thread_wrapper():
                try:
                    # Thread başlangıç
                    thread_info.status = ThreadStatus.RUNNING
                    thread_info.started_at = datetime.now().isoformat()
                    
                    start_time = time.time()
                    
                    # Callback tetikle
                    self._trigger_callback("thread_started", thread_info.to_dict())
                    
                    # Ana fonksiyonu çalıştır
                    result = func(*args, **kwargs)
                    
                    # Thread bitiş
                    end_time = time.time()
                    thread_info.runtime_seconds = end_time - start_time
                    thread_info.status = ThreadStatus.FINISHED
                    thread_info.finished_at = datetime.now().isoformat()
                    
                    # Profiling güncelle
                    self.profiler.update_profile(tid, execution_time=thread_info.runtime_seconds)
                    
                    # Callback tetikle
                    self._trigger_callback("thread_finished", thread_info.to_dict())
                    
                    return result
                    
                except Exception as e:
                    # Hata durumu
                    thread_info.status = ThreadStatus.CRASHED
                    thread_info.error_message = str(e)
                    thread_info.finished_at = datetime.now().isoformat()
                    
                    self.logger.error(f"Thread {tid} crashed: {e}")
                    self._trigger_callback("thread_crashed", thread_info.to_dict())
                    
                finally:
                    # Temizlik
                    self._cleanup_thread(tid)
            
            # Thread oluştur ve başlat
            thread = threading.Thread(target=thread_wrapper, name=thread_info.name)
            thread.daemon = True
            thread.start()
            
            # Referansı sakla
            self.thread_objects[tid] = thread
            
            self.logger.info(f"Thread started: {thread_info.name} ({tid})")
            return True
            
        except Exception as e:
            thread_info.status = ThreadStatus.CRASHED
            thread_info.error_message = str(e)
            self.logger.error(f"Failed to start thread {tid}: {e}")
            return False
    
    def freeze_thread(self, tid: str) -> bool:
        """Thread'i dondur (snapshot al)"""
        if tid not in self.threads:
            return False
        
        thread_info = self.threads[tid]
        
        try:
            # Snapshot oluştur
            if self.snapshots.create_snapshot(tid, thread_info):
                thread_info.status = ThreadStatus.FROZEN
                self._trigger_callback("thread_frozen", thread_info.to_dict())
                self.logger.info(f"Thread frozen: {tid}")
                return True
            
            return False
            
        except Exception as e:
            self.logger.error(f"Failed to freeze thread {tid}: {e}")
            return False
    
    def kill_thread(self, tid: str) -> bool:
        """Thread'i öldür"""
        if tid not in self.threads:
            return False
        
        thread_info = self.threads[tid]
        
        try:
            # Thread objesini al
            thread = self.thread_objects.get(tid)
            if thread and thread.is_alive():
                # Python'da thread'i zorla öldürmenin güvenli yolu yok
                # Sadece durumu güncelle
                thread_info.status = ThreadStatus.CANCELLED
                thread_info.finished_at = datetime.now().isoformat()
                
                self.logger.warning(f"Thread marked as cancelled: {tid} (Python threads cannot be forcefully killed)")
                return True
            
            return False
            
        except Exception as e:
            self.logger.error(f"Failed to kill thread {tid}: {e}")
            return False
    
    def get_thread_info(self, tid: str) -> Optional[ThreadInfo]:
        """Thread bilgisi al"""
        return self.threads.get(tid)
    
    def get_all_threads(self) -> List[ThreadInfo]:
        """Tüm thread'leri al"""
        return list(self.threads.values())
    
    def get_running_threads(self) -> List[ThreadInfo]:
        """Çalışan thread'leri al"""
        return [t for t in self.threads.values() if t.status == ThreadStatus.RUNNING]
    
    def get_app_threads(self, app_id: str) -> List[ThreadInfo]:
        """Uygulama thread'lerini al"""
        return [t for t in self.threads.values() if t.app_id == app_id]
    
    def send_message(self, from_tid: str, to_tid: str, message_type: str, payload: Any) -> bool:
        """Thread'ler arası mesaj gönder"""
        return self.messaging.send_message(from_tid, to_tid, message_type, payload)
    
    def receive_message(self, tid: str, timeout: float = 0.1) -> Optional[ThreadMessage]:
        """Thread mesajı al"""
        return self.messaging.receive_message(tid, timeout)
    
    def start_watchdog(self):
        """Watchdog başlat"""
        if self.watchdog_running:
            return
        
        self.watchdog_running = True
        self.watchdog_thread = threading.Thread(target=self._watchdog_loop, daemon=True)
        self.watchdog_thread.start()
        self.logger.info("Thread watchdog started")
    
    def stop_watchdog(self):
        """Watchdog durdur"""
        self.watchdog_running = False
        if self.watchdog_thread and self.watchdog_thread.is_alive():
            self.watchdog_thread.join(timeout=2.0)
        self.logger.info("Thread watchdog stopped")
    
    def _watchdog_loop(self):
        """Watchdog döngüsü"""
        while self.watchdog_running:
            try:
                self._check_threads()
                time.sleep(self.watchdog_interval)
            except Exception as e:
                self.logger.error(f"Thread watchdog error: {e}")
                time.sleep(10.0)
    
    def _check_threads(self):
        """Thread'leri kontrol et"""
        for tid, thread_info in self.threads.items():
            try:
                # Thread objesi kontrolü
                thread = self.thread_objects.get(tid)
                
                if thread_info.status == ThreadStatus.RUNNING:
                    if not thread or not thread.is_alive():
                        # Thread öldü ama durumu güncellenmemiş
                        thread_info.status = ThreadStatus.CRASHED
                        thread_info.error_message = "Thread died unexpectedly"
                        thread_info.finished_at = datetime.now().isoformat()
                        
                        self.logger.warning(f"Dead thread detected: {tid}")
                        self._trigger_callback("thread_crashed", thread_info.to_dict())
                
                # Çok uzun süre çalışan thread'ler
                if (thread_info.status == ThreadStatus.RUNNING and 
                    thread_info.started_at):
                    
                    try:
                        started = datetime.fromisoformat(thread_info.started_at)
                        runtime = (datetime.now() - started).total_seconds()
                        
                        if runtime > 3600:  # 1 saat
                            self.logger.warning(f"Long running thread detected: {tid} ({runtime:.1f}s)")
                    except Exception:
                        pass
                        
            except Exception as e:
                self.logger.error(f"Error checking thread {tid}: {e}")
    
    def _cleanup_thread(self, tid: str):
        """Thread temizliği"""
        try:
            # Mesajlaşmadan çıkar
            self.messaging.unregister_thread(tid)
            
            # Profiling bitir
            profile = self.profiler.finish_profiling(tid)
            if profile:
                self.logger.debug(f"Thread profile: {tid} - {profile.execution_time:.2f}s")
            
        except Exception as e:
            self.logger.error(f"Thread cleanup error for {tid}: {e}")
    
    def _trigger_callback(self, event_type: str, data: Dict):
        """Callback tetikle"""
        for callback in self.callbacks.get(event_type, []):
            try:
                callback(data)
            except Exception as e:
                self.logger.error(f"Thread callback error for {event_type}: {e}")
    
    def add_callback(self, event_type: str, callback: Callable):
        """Callback ekle"""
        if event_type in self.callbacks:
            self.callbacks[event_type].append(callback)
    
    def get_thread_stats(self) -> Dict:
        """Thread istatistikleri"""
        try:
            status_counts = {}
            for thread_info in self.threads.values():
                status = thread_info.status.value
                status_counts[status] = status_counts.get(status, 0) + 1
            
            type_counts = {}
            for thread_info in self.threads.values():
                ttype = thread_info.thread_type.value
                type_counts[ttype] = type_counts.get(ttype, 0) + 1
            
            return {
                "total_threads": len(self.threads),
                "status_counts": status_counts,
                "type_counts": type_counts,
                "queue_stats": self.queue.get_stats(),
                "pool_stats": self.pool.get_stats(),
                "active_snapshots": len(self.snapshots.snapshots),
                "active_profiles": len(self.profiler.profiles)
            }
            
        except Exception as e:
            self.logger.error(f"Failed to generate thread stats: {e}")
            return {}
    
    def shutdown(self):
        """Modül kapatma"""
        self.logger.info("Shutting down thread manager...")
        
        # Watchdog durdur
        self.stop_watchdog()
        
        # Thread pool kapat
        self.pool.shutdown(wait=True)
        
        # Aktif thread'leri iptal et
        for tid in list(self.threads.keys()):
            if self.threads[tid].status == ThreadStatus.RUNNING:
                self.kill_thread(tid)
        
        self.logger.info("Thread manager shutdown completed")

# Kolaylık fonksiyonları
_thread_manager = None

def init_thread_manager(kernel=None):
    """Thread manager'ı başlat"""
    global _thread_manager
    _thread_manager = ThreadManager(kernel)
    return _thread_manager

def create_thread(func: Callable, *args, **kwargs) -> str:
    """Hızlı thread oluştur"""
    if _thread_manager:
        return _thread_manager.create_thread(func, args, kwargs)
    return ""

def start_thread(tid: str, func: Callable, *args, **kwargs) -> bool:
    """Thread başlat"""
    if _thread_manager:
        return _thread_manager.start_thread(tid, func, args, kwargs)
    return False

def get_thread_manager() -> Optional[ThreadManager]:
    """Thread manager'ı al"""
    return _thread_manager 


DEFAULT_LINE_LENGTH = 88
DEFAULT_EXCLUDES = r"/(\.direnv|\.eggs|\.git|\.hg|\.ipynb_checkpoints|\.mypy_cache|\.nox|\.pytest_cache|\.ruff_cache|\.tox|\.svn|\.venv|\.vscode|__pypackages__|_build|buck-out|build|dist|venv)/"  # noqa: B950
DEFAULT_INCLUDES = r"(\.pyi?|\.ipynb)$"
STDIN_PLACEHOLDER = "__BLACK_STDIN_FILENAME__"



"""
PyCloud OS File System Package
Alt modüller: UserFS, MountManager, FileSearchEngine, PyCloudVFS
"""

from .userfs import UserFS
from .mount import MountManager
from .search import FileSearchEngine
from .vfs import PyCloudVFS, VFSPermission, VFSMount, AppProfile

# Ana fs_main.py modülünden fonksiyonları import et
import sys
from pathlib import Path

# Parent directory'yi path'e ekle
sys.path.insert(0, str(Path(__file__).parent.parent))

# fs_main.py'den gerekli fonksiyonları import et
from fs_main import FileSystem, init_fs, get_file_system

__all__ = [
    'UserFS',
    'MountManager',
    'FileSearchEngine',
    'PyCloudVFS',
    'VFSPermission',
    'VFSMount',
    'AppProfile',
    'FileSystem',
    'init_fs',
    'get_file_system'
] 